#!/bin/sh
# the next line restarts using wish \
exec wish "$0" -- "$@"
# Copyright 2001 Bryan Schofield
# This file is part of RoboTournament.
# 
# RoboTournament is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RoboTournament is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RoboTournament; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

set version "11.13.01"
set cwd [pwd]
set script [info script]
while {[file type $script] == "link"} {
    cd [file dirname $script]
    set script [file readlink $script]
}
cd [file dirname $script]
set homeDir [pwd]
cd $cwd
set libDir [file join $homeDir lib]
set mapDir [file join $homeDir maps]
set imageDir [file join $homeDir images]
set tileDir [file join $imageDir tiles]
set cardDir [file join $imageDir cards]
set weaponDir [file join $imageDir weapons]
set botDir [file join $imageDir bots]
set imageDirs [list $libDir $imageDir $tileDir $cardDir $weaponDir $botDir]
proc MakeImages {dir} {
    if {![file exists $dir]} {
        puts "RT ERROR: Directory does not exist: $dir"
        exit
    }
    set files [glob -nocomplain  -directory $dir *]
    foreach file $files {
        if {[file isdirectory $file]} {
            MakeImages $file 
            continue
        }
        if {[file extension $file] == ".gif"} {
            set image "img-[file rootname [file tail $file]]"
            image create photo $image -file $file
        }
    }
}
MakeImages $imageDir



lappend auto_path $libDir
unset cwd







#
# Required Packages
#
package require RTCSocket
package require RTCLogin
package require RTCGui
package require RTCGameParam
package require RTCError

font create fnt-big \
    -family "Courier" \
    -size "16"
font create fnt-small \
    -family "Courier" \
    -size "12"
font create fnt-tiny \
    -family "Courier" \
    -size "10"

# set background "\#002000"
# set darkBackground "\#001a00"
# set highlight "\#50ff00"
# set foreground "\#c0ffb0"
# set darkForeground "\#508f40"
# set contrastForeground "\#ffa000"
set background "\#004000"
set darkBackground "\#003000"
set highlight "\#50ff00"
set foreground "\#c0ffb0"
set darkForeground "\#508f40"
set contrastForeground "\#ffa000"
option add *Background $background
option add *HighlightBackground $background
option add *HighlightForeground $highlight
option add *Foreground $foreground 
option add *Relief flat
option add *Font "fnt-big"
option add *Button.HighlightThickness 1
option add *Button.HighlightBackground "black"
option add *Button.relief "flat"
option add *Button.Borderwidth 10
option add *Entry.HighlightThickness 1
option add *Entry.HighlightBackground "black"
option add *Radiobutton.HighlightThickness 1
option add *Radiobutton.HighlightBackground "black"
option add *Checkbutton.HighlightThickness 1
option add *Checkbutton.HighlightBackground "black"


namespace eval RTC {
    variable turn
    variable phase
    variable bot 0
}

# Available commands
# rtcsocket open <host> <port>
# rtcsocket send <message>
# rtcsocket close


proc RTC::Init {} {
    wm withdraw .
    if {![rtclogin]} { return 0 }
    rtcgui construct


    return 1
}

proc RTC::Exit {{force 0}} {
    set force 1
    if {$force == 0} {
        set doit [tk_messageBox \
                      -title "RoboTournament" \
                      -icon question \
                      -message "Are you sure you want to quit?" \
                      -type "yesno"]
        if {$doit == "no" } { return }
    }
    exit
}

proc RTC::MessageHandler {message} {
    set message [join $message]
    set type [lindex [split $message] 0]
    set message [join [lrange [split $message] 1 end]]
    switch -- [string tolower $type] {
        botupdate {
            set RTC::turn [lindex $message 0]
            set RTC::phase [lindex $message 1]
            set message [join [lrange $message 2 end]]
            #rtcarena clearrobots
            foreach {bot x y face health frags} $message {
                rtcbotstatus sethealth $bot $health
                rtcbotstatus setfrags $bot $frags
                rtcarena placebot $bot $x $y $face
            }
        }
        setflag   { rtcbotstatus setflag \
                        [lindex $message 0] [lindex $message 1]}
        setteam   {rtcbotstatus setteam \
                        [lindex $message 0] [lindex $message 1]}
        blownup   { rtcarena blowupbot $message }
        crushed   { rtcarena crushbot $message }
        fellinpit { rtcarena fallinpitbot $message }
        cards {
            for {set i 0} {$i < 5} {incr i} {
                rtcregister unlock $i
            }
            set cardsDealt [llength $message]
            if {$cardsDealt < 5} {
                for {set i 0} {$i < $cardsDealt} {incr i} {
                    rtcregister removecard $i
                }
                for {set i $cardsDealt} {$i < 5} {incr i} {
                    if {[rtcregister getcard $i] == ""} {
                        set cards [list m1 m2 m3 bu jp tr tl ut no]
                        set randomCard \
                            [lindex $cards \
                                 [expr int(rand() * [llength $cards])]]
                        rtcregister setcard $i $randomCard
                    }
                    rtcregister lock $i
               }
            } else {
                for {set i 0} {$i < 5} {incr i} {
                    rtcregister removecard $i
               } 
            }
            foreach card [lsort -dictionary $message] {
                rtccardstorage addcard $card
            }
            rtcregister enable
            rtcarena findbot $RTC::bot
        }
        chat { rtcchat add $message }
        servermsg { rtcservermsg add $message }
        removeitem {rtcarena removeitem $message }
        moveitem {rtcarena moveitem $message }
        placeitem {rtcarena placeitem $message }
        gameover {RTC::GameOver $message}
        lasers { rtcarena drawlasers $message }
        map { rtcgui drawarena [string trimleft $message "MAP"] }
        names {
            foreach {bot name} $message {
                rtcbotstatus setname $bot $name
            }
        }
        needmap { rtcgameparam send }
        wanttostart { rtcgui startcontroller }
        wanttopowerup { rtcgui powerupcontroller }
        weapons { rtcregister setweapons $message }
        randomize {
            set locked ""
            for {set i 0} {$i < 5} {incr i} {
                if { [rtcregister islocked $i] } {
                    lappend locked $i
                    rtcregister unlock $i
                }
                rtcregister removecard $i
            }
            foreach card [split $message] {
                rtcregister addcard $card
            }
            foreach i $locked {
                rtcregister lock $i
            }
        }
        runturn { 
            #for {set i 0} {$i < 5} {incr i} {
            #    rtcregister lock $i
            #}
            for {set i 0} {$i < 8} {incr i} {
                rtcbotstatus poweron $i
            }
            foreach bot [split $message] {
                rtcbotstatus poweroff $bot 
            }
            set RTC::phase 0
            rtcgui turnstat 1 
        }
        endturn {
            rtcgui turnstat 0 
        }
        youarebot { set RTC::bot $message }
        default {
            puts "RX: $type $message"
        }
    }
}



#========================================================================
#========================================================================
#========================================================================

proc RTC::GameOver {message} {
    tk_messageBox \
        -title "RoboTournament" \
        -icon info \
        -message "GAME OVER\n$message"
    #exit
}

proc RTC::PickMap {} {



    #
    # Fool the file selection bax to have a brown background,
    # This way we can actually read the text...
    #
    option add *Canvas.background #6e6e42
    while {1} {
        set filetypes {
            {"RoboTournament Map" {.map}} 
            {"All Files" {*}}
        }
        set map [tk_getOpenFile \
                     -title "Select Map" \
                     -filetypes $filetypes]
        if {$map == ""} { continue }
        option add *Canvas.background $::background
        set f [open $map r]
        set data ""
        while {[gets $f line] >= 0} {
            if {([llength $line] == 3) 
                && ([string first $line "\#"] == -1)} {
                set data "$data $line"
            }
        }
        close $f
        rtcsocket send "MAP $data"
        return
    }
}


#========================================================================
#========================================================================
#========================================================================
#========================================================================
#========================================================================
if {![RTC::Init]} { exit }
