#! /bin/sh
# The next line restarts using wish \
exec wish8.0 "$0" "$@"

# Copyright 2001 Bryan Schofield
# This file is part of RoboTournament.
# 
# RoboTournament is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RoboTournament is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RoboTournament; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



set me(version) 0.2                     ;# current version of me
set me(me)      [info script]           ;# the name of this script
if {[file type $me(me)] == "link"} {
    set me(me) [file readlink $me(me)]
}
set dir [pwd]
cd [file dirname $me(me)]
set me(home) [pwd]
cd $dir
set lib  "$me(home)/lib"
set me(tiles)  $me(home)/images/tiles   ;# tiles directory
set me(size)   35                       ;# length the side of a tile
set me(width)  1000                    ;# the maximum width of the map
set me(height) 1000                    ;# the maximum height of the map
set me(map)    .mapf.map_canvas         ;# map canvas widget
set map        $me(map)                 ;# ditto

#source $lib/new_type.tcl
#source $lib/items.tcl

#-------------------------------------------------------------------#
#-------------------------------------------------------------------#
#-------------------------------------------------------------------#
# turn off that #@$! highlight thickness
option add *HighlightThickness 0

rename exit _exit_
proc exit {} {
    if {$Me::m(modified)} {Me::AskToSave}   
    _exit_
}

proc err {msg} {
    puts stderr "$msg"
}

proc set_filename_color {args} {
    switch -- $Me::m(modified) {
        1 {set ::modind " (modified)"}
        0 {set ::modind ""}
    }
}
#-------------------------------------------------------------------#
#-------------------------- Namespace Me ---------------------------#
#-------------------------------------------------------------------#

#--------- The "m" Array ---------#
#
# current_item  - id of the item currently being carried by the 
#                 mouse pointer or empty string
# filename      - name of file currently opened or "Untitled" is not saved
# modified      - indicates if the file has been modified since last save

namespace eval Me {
    variable m
    set m(current_item) ""
    set m(modified)     0
    set m(filename)     Untitled
}


#---------------------------------#
#---------------------------------#
#---------------------------------#
proc Me::MakeTilebar {class} {
    set w .[string tolower $class]
    if {[winfo exists $w]} {
        focus -force $w
        return
    }
    toplevel $w
    wm title $w "$class"
    wm transient $w .
    canvas $w.c \
        -width [expr $::me(size) + 10] \
        -yscrollcommand "$w.ys set"
    scrollbar $w.ys \
        -command "$w.c yview"
    pack $w.c  -side left -fill both -expand 1
    pack $w.ys -side left -fill y
    set y 0
    foreach image [lsort -dictionary $Tiles::class($class)] {
        if {![regexp "_a\[1-9\]" $image]} {
            catch { set i [$w.c create image 5 $y -image $image -anchor nw] }
            incr y 45
            $w.c bind $i <ButtonPress-1> "Me::AddTile $image $class"
        }
    }
    $w.c configure \
        -confine 1 \
        -scrollregion "0 0 [expr $::me(size) + 10] $y"
}

proc Me::AddTile {image class} {
    set x [expr {[$::map canvasx 0] + ($::me(size) / 2)}]
    set y [expr {[$::map canvasy 0] + ($::me(size) / 2)}]
    if {[catch {$::map create image $x $y \
                    -image $image \
                    -anchor nw \
                    -tag "$image tile" \
                }  Me::m(current_item)]} { return }
    $::map raise $Me::m(current_item)
    set i $Me::m(current_item)
    bind $::map <Motion> "Me::MoveAndSnap $Me::m(current_item) %x %y"
    bind $::map <Button-2> "Me::RepeatAddTile $image $class %x %y"
    bind $::map <ButtonRelease-1> Me::DropItem
    bind $::map <Button-1> ""
    $::map configure -cursor hand2
    focus -force $::map
}

proc Me::RepeatAddTile {image class x y} {
    Me::AddTile $image $class
    Me::MoveAndSnap $Me::m(current_item) $x $y
    Me::DropItem
}

#---------------------------------#
#------- Drag and Drop -----------#
#---------------------------------#

#
# returns the id of a tile to modify
# if more than one tile occupies a grid space
# a popup menu if created for the user to 
# select the tile
#
# x - screen x coordinate
# y - screen y coordinate
#
# returns canvas image id
#
proc Me::GetTiles {x y} {
    set cx [$::map canvasx $x]
    set cy [$::map canvasy $y]
    set items [$::map find overlapping $cx $cy [expr {$cx + 1}] [expr {$cy + 1}] ]
    set space ""
    foreach item $items {
        switch -- [lsearch [$::map gettags $item] tile] {
            -1      {}
            default {lappend space $item}
        }
    }
    if {[llength $space] > 1} {
        set m .pickone
        destroy $m
        menu $m -tearoff 0
        foreach image $space {
            $m add command -image [$::map itemcget $image -image] \
                -command "set ::pickme $image"
        }
        tk_popup .pickone [winfo pointerx $::map] [winfo pointery $::map]
        vwait pickme
        set space $::pickme
        return $space
    } else {
        return $space
    }
}

proc Me::DeleteClosest {x y} {
    set Me::m(modified) 1
    set cx [$::map canvasx $x]
    set cy [$::map canvasy $y]
    set items [$::map find overlapping $cx $cy [expr {$cx + 1}] [expr {$cy + 1}] ]
    set space ""
    foreach item $items {
        switch -- [lsearch [$::map gettags $item] tile] {
            -1      {}
            default {lappend space $item}
        }
    }
    $::map delete [lindex $space 0]
}

# x - screen x coordinate
# y - screen y coordinate
proc Me::PickUpClosestItem {x y} {

    variable m

    set space [Me::GetTiles $x $y]
    if {[llength $space] > 0} {
        set m(current_item) [lindex $space 0]
        $::map configure -cursor hand2
        $::map raise $Me::m(current_item)
        bind $::map <Motion> "Me::MoveAndSnap $Me::m(current_item) %x %y"
        bind $::map <ButtonRelease-1> Me::DropItem
        bind $::map <Button-1> ""
    }
}


proc Me::DropItem {} {
    
    variable m

    set m(current_item)    ""
    bind $::map <Motion>   ""
    bind $::map <Button-1> "Me::PickUpClosestItem %x %y"
    if {$::mode != "Delete"} {
        $::map configure -cursor tcross
    }
}


# item - canvas item (tile)
# x    - screen x coordinate
# y    - screen y coordinate
proc Me::MoveAndSnap {item x y} {
    set Me::m(modified) 1
    set x [$::map canvasx $x [expr {int($::me(size) / 2.0)}]]
    set y [$::map canvasy $y [expr {int($::me(size) / 2.0)}]]
    set fx [expr {$x / $::me(size)}]
    set x  [lindex [split $fx . ] 0]
    set fxr [lindex [split $fx . ] 1]
    #if {$fxr > [expr {int($::me(size) / 2.0)}]} {
    #    incr x
    #}
    set fy [expr {int($y / double($::me(size)))}]
    set y  [lindex [split $fy . ] 0]
    set fyr [lindex [split $fy . ] 1]
    #if {$fyr > [expr {int($::me(size) / 2.0)}]} {
    #    incr y
    #}
    $::map coords $item [expr {$x * $::me(size)}] [expr {$y * $::me(size)}]
}


proc Me::RightPopup {x y} {
    set cx [$::map canvasx $x]
    set cy [$::map canvasy $y]
    set items [$::map find overlapping $cx $cy [expr {$cx + 1}] [expr {$cy + 1}] ]
    set space ""
    foreach item $items {
        switch -- [lsearch [$::map gettags $item] grid] {
            -1      {lappend space $item}
        }
    }
    if {[llength $space] < 1} {
        return
    }
    set m $::map.rightPopup
    set r $m.raise
    set l $m.lower
    set d $m.delete
    destroy $m
    menu $m -tearoff 0
    $m add cascade -label "Raise" \
        -menu $r
    $m add cascade -label "Lower" \
        -menu $l
    $m add cascade -label "Delete" \
        -menu $d
    set command($r) "raise"
    set command($l) "lower"
    set command($d) "delete"
    foreach m [list $r $l $d] {
        destroy $m
        menu $m -tearoff 0
        foreach image $space {
            $m add command -image [$::map itemcget $image -image] \
                -command "$::map $command($m) $image ; $::map lower grid ; set Me::m(modified) 1"
        }
    }
    tk_popup $::map.rightPopup [winfo pointerx $::map ] [winfo pointery $::map ]
}

proc Me::LowerTile {x y} {
    set space [Me::GetTiles $x $y]
    if {[llength $space] > 0} {
        $::map lower [lindex $space 0] 
        $::map lower grid
    }
}


#---------------------------------#
#---------- Map Cleaner ----------#
#---------------------------------#
proc Me::MapCleaner {} {
    set tile [$::map find withtag tile]
}


#---------------------------------#
#----- File Input / Output -------#
#---------------------------------#
proc Me::CleanCanvas {} {
    # clean the canvas
    foreach item [$::map find withtag tile] {
        $::map delete $item
    }
}


proc Me::NewMap {} {
    
    variable m
    
    Me::CleanCanvas
    bind $::map <Button-1> "Me::PickUpClosestItem %x %y"
    bind $::map <ButtonPress-3> "Me::RightPopup %x %y"
    set m(filename) Untitled
    set m(modified) 0
}


proc Me::AskToSave {} {
    switch [tk_messageBox -title "Save File?" -type yesno -icon question -message "$Me::m(filename) has been modified.\nDo you want to save the changes?"] {
        yes {Me::SaveMap}
        no  {return}
    }
    return
}

# args:
#   -as - saves map as new file
proc Me::SaveMap {args} {
    
    variable m

    if { $args == "-as" || $m(filename) == "Untitled"} {
        set filetypes {
            {"RoboTournament Map" {.map}} 
            {"All Files" {*}}
        }
        set m(filename) [                \
            tk_getSaveFile               \
            -title "Save RoboTournament Map File" \
            -defaultextension ".map"     \
            -filetypes $filetypes        \
        ]
        if {$m(filename) == ""} {
            set m(filename) Untitled
            return
        }
    } elseif {$m(modified) == 0} {
        return
    }
    set file [open $m(filename) w]
    Me::WriteFile $file
    close $file
    set m(modified) 0
    bind $::map <Button-1> "Me::PickUpClosestItem %x %y"
    bind $::map <ButtonPress-3> "Me::RightPopup %x %y"
}


proc Me::WriteFile {file} {
    # write the file header
    puts $file "# RoboTournament Map File"
    puts $file "# Created by RoboTournament Map Editor $::me(version)\n#"
    puts $file "# File:      $Me::m(filename)"
    foreach tile [$::map find withtag tile] {
        set x [file rootname [lindex [$::map coords $tile] 0] ]
        set y [file rootname [lindex [$::map coords $tile] 1] ]
        set row [expr ($x/$::me(size))]
        set col [expr ($y/$::me(size))]
        set image [$::map itemcget $tile -image]
        set tags [$::map itemcget $tile -tags]
        if {[info exist spot($x,$y)]} {
            switch -- [lsearch $spot($x,$y) $image] {
                -1 {
                    lappend spot($x,$y) $image
                    puts $file "$row $col $image" 
                }
            }
        } else {
            set spot($x,$y) ""
            lappend spot($x,$y) $image
            puts $file "$row $col $image"
        }
    }
}



proc Me::OpenMap {args} {
    if {$Me::m(modified)} {Me::AskToSave}

    if {$args == ""} {
        set filename [tk_getOpenFile]
        if {$filename == ""} { return }
    } else { 
        set filename $args
    }
    Me::CleanCanvas
    # delete all known info about ::grid
    foreach name [array names ::grid] {
        unset ::grid($name)
    }

    set f [open $filename r]
    while {[gets $f line] >= 0} {
        if {[regexp "\#" $line]} { continue }
        set x [expr [lindex $line 0] * $::me(size)]
        set y [expr [lindex $line 1] * $::me(size)]
        set tile [lindex $line 2]
        if {[catch {$::map create image $x $y -image $tile -anchor nw \
                        -tags "$x,$y tile"}]} { continue }
        lappend ::grid($x,$y) $tile
    }


    set Me::m(filename) $filename
    set Me::m(modified) 0
    $::map yview moveto 0
    $::map xview moveto 0
    bind $::map <Button-1> "Me::PickUpClosestItem %x %y"
    bind $::map <ButtonPress-3> "Me::RightPopup %x %y"
    set bbox [$::map bbox tile]
    $::map configure \
        -width [lindex $bbox 2] \
        -height [lindex $bbox 3] 

}

proc Me::OldOpenMap {args} {
        
    if {$Me::m(modified)} {Me::AskToSave}

    if {$args == ""} {
        set filename [tk_getOpenFile]
        if {$filename == ""} { return }
    } else { 
        set filename $args
    }
    Me::CleanCanvas
    set file [open $filename r]
    while {![eof $file]} {
        gets $file line
        if {[llength $line] > 0} {
            if {![regexp "#" $line]} {
                regsub -all "sword" $line "prod" line
                regsub -all "nsblock" $line "udBlock" line
                regsub -all "ewblock" $line "lrBlock" line
                eval $line
            }
        }
    }
    set Me::m(filename) $filename
    set Me::m(modified) 0
    $::map yview moveto 0
    $::map xview moveto 0
    bind $::map <Button-1> "Me::PickUpClosestItem %x %y"
    bind $::map <ButtonPress-3> "Me::RightPopup %x %y"
    set bbox [$::map bbox tile]
    $::map configure \
        -width [lindex $bbox 2] \
        -height [lindex $bbox 3] 

}


#---------------------------------#
#------------- Grid --------------#
#---------------------------------#

proc Me::DrawGrid {} {
    for {set r 0} {$r <= $::me(height)} {incr r $::me(size)} {
        $::map create line $r 0 $r $::me(width) \
            -fill "\#333333" \
            -width 1 \
            -tag grid
    }
    for {set c 0} {$c <= $::me(width)} {incr c $::me(size)} {
        $::map create line 0 $c $::me(height) $c \
            -fill "\#333333" \
            -width 1 \
            -tag grid
    }
    $::map lower grid
}

proc Me::ShowHideGrid {} {
    switch -- $::pref(show_grid) {
        1 {Me::DrawGrid}
        0 {$::map delete grid} 
    }
}
#-------------------------------------------------------------------#
#------------------------- Namespace Tiles -------------------------#
#-------------------------------------------------------------------#
# the "class" array
# class(list) - list of classes i.e. terraine, characters, sprites...
# for every class there will be a list created with the names
#   of every image in the class. ie. class(terraine) will contain 
#   a list like the following: grass grass1 tree_a0 tree_a1 mountain...
#
namespace eval Tiles {
    variable class
    variable tileDirectory
}

proc Tiles::LearnTiles {} {
    set tile_dir [lsort -dictionary [Tiles::GlobDir $Tiles::tileDirectory -nocomplain *]]
    foreach file $tile_dir {
        if {[file isdirectory $Tiles::tileDirectory/$file]} {
            lappend Tiles::class(list) $file
            set Tiles::class($file) ""
            set file_dir [lsort -dictionary [Tiles::GlobDir $Tiles::tileDirectory/$file -nocomplain *.gif]]
            # glob .gif for images
            set Tiles::class($file) ""
            foreach gif $file_dir {
                set tilename [file rootname $gif]
                lappend Tiles::class($file) $tilename

                if {![info exists ::type($tilename,dominant)]} {
                    #puts "Unknown Type: $tilename"
#                     set ui [tk_messageBox \
#                                 -icon warning \
#                                 -title "New Item Type Detected" \
#                                 -type yesno \
#                                 -message "Type \"$tilename\" is not known. Define it in items.tcl by using the item_config.tcl script before running XSoko. Do you want to run it now?"]
#                     switch -- $ui {
#                         yes {eval exec [info nameofexecutable] $::me(home)/item_config.tcl -item $file $tilename & }
#                     }
                }
            }
        }
    }
}

proc Tiles::GlobDir {dir args} {
    set working [pwd]
    cd $dir
    set globber [eval glob $args]
    cd $working
    return "$globber"
}

proc Tiles::MakeImages {} {
    foreach c $Tiles::class(list) {
        foreach image $Tiles::class($c) {
        image create photo $image -file $Tiles::tileDirectory/$c/$image.gif
        }
    }
}
#-------------------------------------------------------------------#
#---------------------------- Images -------------------------------#
#-------------------------------------------------------------------#
set modified_indicator {
 	#define modified_indicator.icn_width 8
 	#define modified_indicator.icn_height 8
 	static unsigned char modified_indicator.icn_bits = {
		0x3c, 0x7e, 0xff, 0xff, 0xff, 0xff, 0x7e, 0x3c
	}
}
image create bitmap modified_indicator -data [set modified_indicator] 



#-------------------------------------------------------------------#
#---------------------------- Menubar ------------------------------#
#-------------------------------------------------------------------#
#----------- .menubar ------------#
set m .menubar
menu $m -type menubar
$m add cascade -label "File" \
    -underline 0 \
    -menu .file
$m add cascade -label "TileBars" \
    -underline 0 \
    -menu .tilebars
$m add cascade -label "Options" \
    -underline 0 \
    -menu .options

#------------ .file --------------#
set m .file
menu $m -tearoff 0
$m add command -label "New" \
    -underline 0 \
    -command "Me::NewMap"
$m add command -label "Open..." \
    -underline 0 \
    -command "Me::OpenMap"
# $m add command -label "Open Old Map..." \
#    -underline 0 \
#   -command "Me::OldOpenMap"
$m add separator
$m add command -label "Save" \
    -underline 0 \
    -command "Me::SaveMap"
$m add command -label "Save As..." \
    -underline 5 \
    -command "Me::SaveMap -as"
$m add command -label "Write to Stderr" \
    -command "Me::WriteFile stderr"
$m add separator
$m add command -label "Exit" \
    -underline 1 \
    -command "exit"
#---------- .tilebars -----------#
set m .tilebars
menu $m \
    -tearoff 1 \
    -title "TileBars"
#----------- .options -----------#
set m .options
menu $m \
    -tearoff 1 \
    -title Options
$m add checkbutton -label "Show Grid" \
    -underline 5 \
    -variable ::pref(show_grid) \
    -onvalue 1 \
    -offvalue 0 \
    -command "Me::ShowHideGrid"
# $m add separator

#-------------------------------------------------------------------#
#------------------------------ Main -------------------------------#
#-------------------------------------------------------------------#



wm title . "RoboTournament Map Editor ($me(version))"
wm protocol . WM_DELETE_WINDOW exit
. configure -menu .menubar

#---------------------------------#
#-------- The Map Canvas ---------#
#---------------------------------#
set f .mapf
frame $f
scrollbar $f.ys \
    -command "$map yview"
scrollbar $f.xs \
    -command "$map xview" \
    -orient horizontal
canvas $map \
    -yscrollcommand "$f.ys set" \
    -xscrollcommand "$f.xs set" \
    -confine 1 \
    -scrollregion "0 0 $me(width) $me(height)" \
    -cursor tcross
grid $map  -col 0 -row 0 -sticky news 
grid $f.ys -col 1 -row 0 -sticky ns
grid $f.xs -col 0 -row 1 -sticky ew   
grid columnconfigure $f 0 -weight 1
grid rowconfigure    $f 0 -weight 1

#---------------------------------#
#---------------------------------#
#---------------------------------#
set f .status
frame $f
label $f.filename \
    -textvariable Me::m(filename)
label $f.modified \
    -textvariable ::modind
pack $f.filename -side left
pack $f.modified -side left

trace variable Me::m(modified) w "set_filename_color"    
#---------------------------------#
#---------------------------------#
#---------------------------------#

grid .mapf   -col 0 -row 0 -sticky news
grid .status -col 0 -row 1 -sticky  ew
grid columnconfigure . 0 -weight 1
grid rowconfigure    . 0 -weight 1

#---------------------------------#
#---------------------------------#
#---------------------------------#

bind $::map <Shift-Button-1> "Me::DeleteClosest %x %y ; break"
bind . <Shift_L> "set ::mode Delete ; $::map configure -cursor X_cursor"
bind . <KeyRelease-Shift_L> "set ::mode {} ; $::map configure -cursor tcross"
set ::mode {}

# learn the tiles
set Tiles::tileDirectory $::me(tiles)
Tiles::LearnTiles
Tiles::MakeImages
# add tilebar command to the menu
foreach c $Tiles::class(list) {
    .tilebars add command -label "$c" \
        -command "Me::MakeTilebar $c"
}

set help {
    map_editor.tcl [-convert] [-old] [map_file]
}

set ::convert 0
set ::old 0
# process command line arguments
if {$argc > 0} {
    for {set i 0} {$i < [llength $argv]} {incr i} {
        switch -regexp -- [lindex $argv $i]  {
            -h.*    {puts stderr "$help" ; return}
            -c.*    {set ::convert 1 ; set ::old 1}
            -o.*    {set ::old 1}
            default {set map_file [lindex $argv $i]}
        }
    }
    if {[file exists $map_file]} {
        if {$::old} {
            Me::OldOpenMap $map_file
            if {$::convert} {
                Me::WriteFile stdout
                exit
            }
        } else {
            Me::OpenMap $map_file
        }
    }
} else {
    Me::NewMap
}
