# Copyright 2001 Bryan Schofield
# This file is part of RoboTournament.
# 
# RoboTournament is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RoboTournament is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RoboTournament; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package provide RTCArena 1.0

proc ::rtcarena {subcommand args} {
    set help "\nShould be:\n\trtcarena construct <path>"
    #
    # subcommands that do not require register number
    #
    switch -- [string tolower $subcommand] {
        blowupbot {
            return [RTCArena::BlowUpBot $args]
        }
        construct {
            return [RTCArena::Construct $args]
        }
        clearrobots {
            return [$RTCArena::canvas delete robot]
        }
        crushbot {
            return [RTCArena::CrushBot $args]
        }
        draw {
            return [RTCArena::Draw [join $args]]
        }
        drawlasers {
            return [RTCArena::DrawLasers [join $args]]
        }
        fallinpitbot {
            return [RTCArena::FallInPitBot $args]
        }
        placebot {
            return [eval RTCArena::PlaceBot $args]
        }
        findbot {
            return [eval RTCArena::FindBot $args]
        }
        removeitem {
            return [eval RTCArena::RemoveItem $args]
        }
        moveitem {
            return [eval RTCArena::PlaceItem [lindex $args 0] \
                        [lindex $args 1] [lindex $args 2]]
        }
        placeitem {
            return [eval RTCArena::PlaceItem [lindex $args 0] \
                        [lindex $args 1] [lindex $args 2]]
        }
        default {
            puts stderr "Unkown command: rtcarena $subcommand $args"
        }
    }

}

namespace eval RTCArena {
    variable canvas
    variable size 35
    variable findAfter ""
    variable itemStorage
    variable laserTimer 750
}

proc RTCArena::Construct { w } {
    
    frame $w \
        -relief flat \
        -borderwidth 0 \
        -background "$::darkBackground" \
        -highlightthickness 1 \
        -highlightbackground "black"

    set RTCArena::canvas [canvas $w.c \
                              -borderwidth 0 \
                              -background "$::darkBackground" \
                              -takefocus 0 \
                              -highlightthickness 0 \
                              -yscrollcommand "$w.y set" \
                              -xscrollcommand "$w.x set"]
    scrollbar $w.y \
        -command "$w.c yview" \
        -relief flat \
        -borderwidth 0 \
        -troughcolor "$::darkBackground" \
        -background "black" \
        -highlightthickness 0 \
        -highlightbackground "$::background" \
        -elementborderwidth 0 \
        -width 9\
        -takefocus 0 \
        -activebackground "$::darkForeground"
    scrollbar $w.x \
        -command "$w.c xview" \
        -relief flat \
        -borderwidth 0 \
        -troughcolor "$::darkBackground" \
        -background "black" \
        -highlightthickness 0 \
        -highlightbackground "$::background" \
        -elementborderwidth 0 \
        -width 9\
        -takefocus 0 \
        -orient "horizontal" \
        -activebackground "$::darkForeground"





    grid $w.c -row 0 -col 0 -stick news
    grid $w.y -row 0 -col 1 -stick ns -padx 2 -pady 2
    grid $w.x -row 1 -col 0 -stick ew -padx 2 -pady 2
    grid rowconfigure $w 0 -weight 1
    grid columnconfigure $w 0 -weight 1

    return $w
}


proc RTCArena::ConvertCoord { coord } {
    return [expr $coord * $RTCArena::size]
}
proc RTCArena::Draw { mapData } {
    set mapData [join $mapData]
    variable size
    set images [image names]
    foreach {x y i image} [split [string trim $mapData]] {
        #puts "$x $y $i $image"
        if {[lsearch $images "img-$image"] == -1} {
            puts "Unknown Tile: $image"
            continue
        }
        set x [RTCArena::ConvertCoord $x]
        set y [RTCArena::ConvertCoord $y]
        $RTCArena::canvas create image $x $y \
            -image "img-$image" \
            -anchor nw \
            -tag "tile $i type-[string index $image 0]"
    }
    $RTCArena::canvas configure \
        -confine 1 \
        -scrollregion [$RTCArena::canvas bbox tile]

    $RTCArena::canvas raise "type-f"
    $RTCArena::canvas raise "type-p" 
    $RTCArena::canvas raise "type-b" 
    $RTCArena::canvas raise "type-c" 
    $RTCArena::canvas raise "type-d"
    $RTCArena::canvas raise "type-g"
    $RTCArena::canvas raise "type-t"
    $RTCArena::canvas raise "type-r" 
    $RTCArena::canvas raise "type-o"
    $RTCArena::canvas raise "type-w"
    $RTCArena::canvas raise "type-l"
    $RTCArena::canvas raise "type-z"
    $RTCArena::canvas raise "type-u"
    $RTCArena::canvas raise "type-a"
    $RTCArena::canvas raise "type-h" 

    return 1
}

proc RTCArena::PlaceBot { botnum x y face} {
    regsub "^bot" $botnum "" botnum
    $RTCArena::canvas delete "bot$botnum"
    set x [RTCArena::ConvertCoord $x]
    set y [RTCArena::ConvertCoord $y]
    $RTCArena::canvas create image $x $y \
        -image img-bot-$botnum-$face \
        -anchor nw \
        -tag "robot bot$botnum"
    $RTCArena::canvas lower "bot$botnum" "type-h" 
}

proc RTCArena::FindBot { botnum } {
    regsub "^bot" $botnum "" botnum
    set item [$RTCArena::canvas find withtag "bot$botnum"]
    if {$item == ""} { bell ; return }
    after cancel $RTCArena::findAfter
    $RTCArena::canvas delete "bot-finder"
    eval $RTCArena::canvas create image \
        [$RTCArena::canvas coords "bot$botnum"] \
        -image "img-bot-finder" \
        -anchor nw \
        -tag "bot-finder"
    set RTCArena::findAfter \
        [after 2000 {$RTCArena::canvas delete "bot-finder"}]
}

proc RTCArena::DrawLasers { laserPkt } {
    variable size
    foreach {src sx sy ex ey} [split [string trim $laserPkt]] {
        switch -- $src {
            bot0 { set color "\#FFFF00" ; set offset 1}
            bot1 { set color "\#00FF00" ; set offset -1}
            bot2 { set color "\#FF0000" ; set offset 2}
            bot3 { set color "\#BF00BF" ; set offset -2}
            bot4 { set color "\#0000FF" ; set offset 3}
            bot5 { set color "\#FF8000" ; set offset -3}
            bot6 { set color "\#00FFFF" ; set offset 4}
            bot7 { set color "\#FF7FFF" ; set offset -4}
            default { set color "\#C9FF00" ; set offset 0}
        }
        if {[catch {$RTCArena::canvas create line \
                        [expr ($sx * $size)+($size/2.0)+$offset] \
                        [expr ($sy * $size)+($size/2.0)+$offset] \
                        [expr ($ex * $size)+($size/2.0)+$offset] \
                        [expr ($ey * $size)+($size/2.0)+$offset] \
                        -fill $color \
                        -width 1 \
                        -capstyle "round" \
                        -tag "laser"} err]} {
            puts "ERROR: Bad laser data: $src $sx $sy $ex $ey\n$err"
        }
    }
    $RTCArena::canvas raise "laser"
    if {[$RTCArena::canvas find withtag "type-l"] != ""} {
        $RTCArena::canvas lower "laser" "type-l"
    } else {
        $RTCArena::canvas lower "laser" "robot"
    }
    after $RTCArena::laserTimer { $RTCArena::canvas delete "laser" }
}

proc RTCArena::BlowUpBot { botnum } {
    regsub "^bot" $botnum "" botnum
    rtcbotstatus sethealth $botnum 0
    set image [lindex [list img-bot-$botnum-crushed \
                           img-bot-$botnum-exploded] \
                   [expr int(rand() * 2)]]
    $RTCArena::canvas itemconfigure "bot$botnum" \
        -image "$image"
    #puts "    RTCArena::BlowUpBot $botnum "
    update
}

proc RTCArena::CrushBot { botnum } {
    regsub "^bot" $botnum "" botnum
    rtcbotstatus sethealth $botnum 0
    set image [lindex [list img-bot-$botnum-crushed \
                           img-bot-$botnum-exploded] \
                   [expr int(rand() * 2)]]
    $RTCArena::canvas itemconfigure "bot$botnum" \
        -image "$image"
    #puts "     RTCArena::CrushBot $botnum "
    update
}

proc RTCArena::FallInPitBot { botnum } {
    regsub "^bot" $botnum "" botnum
    $RTCArena::canvas delete "bot$botnum"
    rtcbotstatus sethealth $botnum 0
    #puts "    RTCArena::FallInPitBot $botnum "
    update
}

proc RTCArena::RemoveItem {itemId} {
    if {[string index $itemId 0] != "t"} {
        set itemId "t$itemId"
    }
    set RTCArena::itemStorage($itemId) \
        [$RTCArena::canvas itemcget $itemId -image]
    set RTCArena::itemStorage($itemId,tags) \
        [$RTCArena::canvas itemcget $itemId -tag]
    $RTCArena::canvas delete $itemId
}
proc RTCArena::PlaceItem {itemId x y} {
    if {[string index $itemId 0] != "t"} {
        set itemId "t$itemId"
    }
    set x [RTCArena::ConvertCoord $x]
    set y [RTCArena::ConvertCoord $y]
    if {[$RTCArena::canvas find withtag $itemId] == ""} {
        if {[info exists RTCArena::itemStorage($itemId)]} {
            $RTCArena::canvas create image $x $y \
                -image "$RTCArena::itemStorage($itemId)" \
                -anchor nw \
                -tag "$RTCArena::itemStorage($itemId,tags)"
        } else {
            puts stderr "ERROR: Arena has no record of item $itemId"
        }
    } else {
       $RTCArena::canvas coord $itemId $x $y 
    }
}
