# Copyright 2001 Bryan Schofield
# This file is part of RoboTournament.
# 
# RoboTournament is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RoboTournament is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RoboTournament; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package provide RTCBotStatus 1.0

proc ::rtcbotstatus {subcommand args} {
    set help "\nShould be:\n\trtcbotstatus construct <path>" 
    if {[string tolower $subcommand] == "construct"} {
        return [RTCBotStatus::Construct $args]
    }
    switch -- [string tolower $subcommand] {
    }
    #
    # subcommands that do require robot number
    #
    regsub "^bot" [lindex $args 0] "" robot
    if {([string is integer $robot] == 0)
        || ($robot < 0) || ($robot > 7)} {
        return -code error \
            "Invalid robot number: \"$robot\" $help"
    }

    switch -- [string tolower $subcommand] {
        name {
            return [RTCBotStatus::Name $robot]
        }
        health {
            return [RTCBotStatus::Health $robot]
        }
        poweron {
            $RTCBotStatus::powerIndicator($robot) configure \
                -image img-power-status-on
            return 1
        }
        poweroff {
            $RTCBotStatus::powerIndicator($robot) configure \
                -image img-power-status-off
            return 1
        }
        setflag {
           $RTCBotStatus::flag($robot) configure \
               -image img-flag-[lindex $args 1]
            return 1
        }
        setfrags {
            return [RTCBotStatus::SetFrags $robot \
                        [format "%1.2f" [lindex $args 1]]]
        }
        sethealth {
            set health [lindex $args 1]
            if {([llength $args] != 2)
                || ([string is integer $health] == 0)
                || ($health < 0) || ($health > 999)} { 
                return -code error \
                    "Invalid Health Parameter: $subcommand $args $help"
            }
            return [RTCBotStatus::SetHealth $robot $health]
        }
        setname {
            return [RTCBotStatus::SetName $robot [join [lrange $args 1 end]]]
        }
        setteam {
            set color(m) "\#c6273f"
            set color(b) "\#009dff"
            $RTCBotStatus::nameLabel($robot) configure \
                -foreground $color([lindex $args 1])
            return
        }
    }

    return -code error "Unknown command: \"$subcommand\" \"$args\" $help"
}


namespace eval RTCBotStatus {
    variable health
    variable name
    variable frags
    variable healthBar
    variable powerIndicator
    variable nameLabel
    variable flag
}

proc RTCBotStatus::Construct { w } {
    frame $w \
        -relief flat \
        -borderwidth 0 \
        -background "$::darkBackground" \
        -highlightthickness 1 \
        -highlightbackground "black"
    set col 1
    set row 1
    grid [frame $w.f1 -height 4 -width  4 -background "$::darkBackground"] \
        -row 0 -col 0
    grid [frame $w.f2 -height 4 -width  4 -background "$::darkBackground" ] \
        -row 8 -col 4
    grid [frame $w.f3 -width 4 -background "$::darkBackground"] \
        -row 0 -col 2
    for {set i 0} {$i < 8} { incr i} {
        set RTCBotStatus::health($i) 100
        set RTCBotStatus::frags($i) 0.00
        set RTCBotStatus::name($i) "Robot $i"
        set f [frame $w.b$i \
                   -relief flat \
                   -borderwidth 0 \
                   -background "$::background" \
                   -highlightthickness 1 \
                   -highlightbackground "black"]
        button $f.image \
            -image img-bot-$i\
            -background "$::background" \
            -relief flat \
            -borderwidth 0 \
            -highlightthickness 0 \
            -anchor nw \
            -activebackground "$::darkForeground" \
            -command "rtcarena findbot bot$i"


        set RTCBotStatus::nameLabel($i) \
            [label $f.name \
                 -textvariable RTCBotStatus::name($i) \
                 -font fnt-tiny \
                 -width 3 \
                 -anchor w\
                 -background "$::background"]
        set RTCBotStatus::powerIndicator($i) \
            [label $f.power \
                 -image img-power-status-on]
        set RTCBotStatus::healthBar($i) \
            [RTCBotStatus::HealthBar  $f.health]

        set RTCBotStatus::flag($i) \
                [label $f.flag \
                     -image img-flag-empty \
                    -anchor center]

        label $f.frags \
            -textvariable RTCBotStatus::frags($i) \
            -font fnt-tiny \
            -width 5\
            -anchor e

        grid $f.image  -row 0 -col 0 -rowspan 2 -padx 2 -pady 2
        grid $f.name   -row 0 -col 1 -stick ew -columnspan 4
        grid $f.power  -row 1 -col 1 -stick w
        grid $f.health -row 1 -col 2 -stick w -padx 1
        grid $f.flag   -row 1 -col 3 -padx 2
        grid $f.frags  -row 1 -col 4 -stick ew
        grid columnconfigure $f {3} -weight 1
        grid rowconfigure $f 0 -weight 1

        grid $f -row $row -col $col -stick ew
        if {$i == 3} {
            set row 1
            incr col 2
        } else {
            incr row 
            grid [frame $w.fi$i -height 4 -background "$::darkBackground" ] \
                -row $row -col 1
            incr row
        }
    }

    grid columnconfigure $w {1 3} -weight 1
    return $w
}


proc RTCBotStatus::HealthBar { w } {
    canvas $w \
        -width 25 \
        -height 6 \
        -borderwidth 0 \
        -highlightthickness 1 \
        -highlightbackground "black"
    $w create image 1 1 \
        -anchor nw \
        -image "img-health-bar" \
        -tag "bg"
    $w create image 26 1 \
        -anchor nw \
        -image "img-health-bar-cover" \
        -tag "cover"
    
    return $w
}

proc RTCBotStatus::Name {robot} {
   return $RTCBotStatus::name($robot)
}

proc RTCBotStatus::Health {robot} {
   return $RTCBotStatus::health($robot)
}
proc RTCBotStatus::Frags {robot} {
   return $RTCBotStatus::frags($robot)
}

proc RTCBotStatus::SetFrags {robot frags} {
    set RTCBotStatus::frags($robot) $frags
    return $RTCBotStatus::frags($robot)
}

proc RTCBotStatus::SetHealth {robot health} {
    set RTCBotStatus::health($robot) $health
    $RTCBotStatus::healthBar($robot) coords "cover" \
        [expr (25 * $health/100.0) + 1] 1
    return $RTCBotStatus::health($robot)
}

proc RTCBotStatus::SetName {robot name} {
    set RTCBotStatus::name($robot) $name
    return $RTCBotStatus::name($robot)
}
