# Copyright 2001 Bryan Schofield
# This file is part of RoboTournament.
# 
# RoboTournament is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RoboTournament is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RoboTournament; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package provide RTCCardStorage 1.0


proc ::rtccardstorage {subcommand args} {
    set help "\nShould be:\n\trtccardstorage construct\n\trtccardstorage setcard <regnum> <card>\n\trtccardstorage getcard <regnum>\n\trtccardstorage removecard <regnum>" 
    #
    # subcommands that do not require register number
    #
    switch -- [string tolower $subcommand] {
        construct {
            return [RTCCardStorage::Construct $args]
        }
        addcard {
            return [RTCCardStorage::AddCard $args]
        }
        startcontroller {
            return [RTCCardStorage::StartController]
        }
        powerupcontroller {
            return [RTCCardStorage::PowerUpController]
        }
        turnstat {
            return [RTCCardStorage::TurnStat $args]
        }
    }

    #
    # subcommands that do require register number
    #
    if {([string is integer [lindex $args 0]] == 0)
        || ([lindex $args 0] < 0) 
        || ([lindex $args 0] > 9)} {
        return -code error \
            "Invalid register number: [lindex $args 0] $help"
    }

    switch -- [string tolower $subcommand] {
        setcard {
            if {[llength $args] != 2} { 
                return -code error \
                    "Not enough arguments: $subcommand $args $help"
            }
            return [RTCCardStorage::SetCard [lindex $args 0] [lindex $args 1]]
        }
        getcard {
            return [RTCCardStorage::GetCard [lindex $args 0]]
        }
        removecard {
            return [RTCCardStorage::RemoveCard [lindex $args 0]]
        }
    }
    return -code error "Unknon command: \"$subcommand\" \"$args\" $help"
}


namespace eval RTCCardStorage {
    variable canvas
    variable x
    variable y
    variable card
}


proc RTCCardStorage::Construct { w } {
    set RTCCardStorage::canvas [canvas $w \
                                    -width 285 \
                                    -height 160 \
                                    -borderwidth 0 \
                                    -highlightthickness 0]
    $w create image 0 0 \
        -anchor nw \
        -image img-card-storage-bg \
        -tag bg

    set x 10
    set y 10
    for { set i 0} {$i < 9} {incr i} {
        set RTCCardStorage::x($i) $x
        set RTCCardStorage::y($i) $y
        incr x 55
        if {$i == 4} {
            set x 37
            incr y 75
        }
        set RTCCardStorage::card($i) ""
    }

    return $w
}

proc RTCCardStorage::AddCard { card } {
    for { set i 0 } { $i < 9 } { incr i} {
        if { $RTCCardStorage::card($i) == "" } {
            RTCCardStorage::SetCard $i $card
            return 1
        }
    }
    return 0
}

proc RTCCardStorage::SetCard {regnum card} {
    RTCCardStorage::RemoveCard $regnum
    if {![regexp {^img-} $card]} {
        set RTCCardStorage::card($regnum) "$card"
        set card img-$card
    } else {
        set RTCCardStorage::card($regnum) [string range 4 end]
    }
    $RTCCardStorage::canvas create image \
        $RTCCardStorage::x($regnum) $RTCCardStorage::y($regnum) \
        -image $card \
        -anchor nw \
        -tag "card-$regnum"
    $RTCCardStorage::canvas bind "card-$regnum" <1> \
        "RTCCardStorage::ClickCard $regnum"
    return 1
}


proc RTCCardStorage::RemoveCard {regnum} {
    $RTCCardStorage::canvas delete "card-$regnum"
    set RTCCardStorage::card($regnum) ""
    return 1
}

proc RTCCardStorage::GetCard {regnum} {
    return $RTCCardStorage::card($regnum)
}

proc RTCCardStorage::ClickCard {regnum} {
    if {[rtcregister addcard [RTCCardStorage::GetCard $regnum]]} {
        RTCCardStorage::RemoveCard $regnum
    }
}


proc RTCCardStorage::StartController {} {
    for { set i 0 } { $i < 9 } { incr i} {
        RTCCardStorage::RemoveCard $i
    }
    button $RTCCardStorage::canvas.start \
        -text "Start" \
        -command "RTCCardStorage::Start $RTCCardStorage::canvas.start" \
        -borderwidth 1 \
        -highlightbackground "black"
    $RTCCardStorage::canvas create window 142 80 \
        -anchor "center" \
        -window "$RTCCardStorage::canvas.start" \
        -tag "startbutton"
    RTCLogin::FlashButton $RTCCardStorage::canvas.start
}

proc RTCCardStorage::Start { button } {
    destroy $button
    $RTCCardStorage::canvas delete "startbutton"
    rtcsocket send "START"
}

proc RTCCardStorage::PowerUpController {} {
    for { set i 0 } { $i < 9 } { incr i} {
        RTCCardStorage::RemoveCard $i
    }
    $RTCCardStorage::canvas create text 142 50 \
        -text "Power Up?" \
        -anchor center \
        -fill $::background \
        -tag "powerup" \
        -font "fnt-big"
    button $RTCCardStorage::canvas.yes \
        -text "Yes" \
        -command "RTCCardStorage::PowerUp $RTCCardStorage::canvas.yes $RTCCardStorage::canvas.no 1" \
        -borderwidth 1 \
        -highlightbackground "black" \
        -width 3
    button $RTCCardStorage::canvas.no \
        -text "No" \
        -command "RTCCardStorage::PowerUp $RTCCardStorage::canvas.yes $RTCCardStorage::canvas.no 0" \
        -borderwidth 1 \
        -highlightbackground "black" \
        -width 3
    $RTCCardStorage::canvas create window 90 90 \
        -anchor "center" \
        -window "$RTCCardStorage::canvas.yes" \
        -tag "yesbutton"
    $RTCCardStorage::canvas create window 194 90 \
        -anchor "center" \
        -window "$RTCCardStorage::canvas.no" \
        -tag "nobutton"
    RTCLogin::FlashButton $RTCCardStorage::canvas.yes
    return 1
}

proc RTCCardStorage::PowerUp {b1 b2 power} {
    destroy $b1
    destroy $b2
    $RTCCardStorage::canvas delete "yesbutton"
    $RTCCardStorage::canvas delete "nobutton"
    $RTCCardStorage::canvas delete "powerup"
    rtcsocket send "POWERUP $power"
}

proc RTCCardStorage::TurnStat {draw} {
    if {$draw == 1} {
        $RTCCardStorage::canvas create window 142 80 \
            -anchor "center" \
            -window [RTCCardStorage::CreateTurnStatFrame \
                         $RTCCardStorage::canvas.turnstat] \
            -tag "turnstat"
    } else {
        $RTCCardStorage::canvas delete "turnstat"
    }
}

proc RTCCardStorage::CreateTurnStatFrame { f } {
    if {[winfo exists $f]} { return $f }
    frame $f \
        -relief flat \
        -borderwidth 0 \
        -background "$::background" \
        -highlightthickness 1 \
        -highlightbackground "black"
    label $f.t -text "Turn"
    label $f.p -text "Phase"
    label $f.te \
        -textvariable "RTC::turn" \
        -width 4 \
        -highlightthickness 1 \
        -highlightbackground "black" \
        -background "$::darkBackground"
    label $f.pe \
        -textvariable "RTC::phase" \
        -width 4 \
        -highlightthickness 1 \
        -highlightbackground "black" \
        -background "$::darkBackground"
    grid $f.t $f.te -stick w -padx 2 -pady 2
    grid $f.p $f.pe -stick w -padx 2 -pady 2
    return $f
}

