# Copyright 2001 Bryan Schofield
# This file is part of RoboTournament.
# 
# RoboTournament is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RoboTournament is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RoboTournament; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package provide RTCChat 1.0

proc ::rtcchat {subcommand args} {
    set help "\nShould be:\n\trtcchat construct <path>" 
    if {[string tolower $subcommand] == "construct"} {
        return [RTCChat::Construct $args]
    }
    switch -- [string tolower $subcommand] {
        add { return [RTCChat::AddText $args] }
    }
    return -code error "Unknown command: \"$subcommand\" \"$args\" $help"
}


namespace eval RTCChat {
    variable sendText ""
    variable text ""
}
proc RTCChat::Construct { w } {
    frame $w \
        -relief flat \
        -highlightthickness 0\
        -borderwidth 0

    frame $w.text \
        -relief flat \
        -borderwidth 0 \
        -background "$::darkBackground" \
        -highlightthickness 1 \
        -highlightbackground "black"
    set RTCChat::text [text $w.text.t \
                           -yscrollcommand "$w.text.y set" \
                           -wrap word \
                           -font fnt-tiny \
                           -width 20 \
                           -height 6 \
                           -state disabled \
                           -highlightthickness 0 \
                           -background "$::darkBackground" \
                           -takefocus 0 \
                           -cursor {}]
    scrollbar $w.text.y \
        -command "$w.text.t yview" \
        -relief flat \
        -borderwidth 0 \
        -troughcolor "$::darkBackground" \
        -background "black" \
        -highlightthickness 0 \
        -highlightbackground "$::background" \
        -elementborderwidth 0 \
        -width 9 \
        -takefocus 0 \
        -activebackground "$::darkForeground"
    grid $w.text.t -row 0 -col 0 -stick news
    grid $w.text.y -row 0 -col 1 -stick ns -padx 2 -pady 2
    grid columnconfigure $w.text 0 -weight 1
    grid rowconfigure    $w.text 0 -weight 1
    

    frame $w.mesg \
        -relief flat \
        -borderwidth 0
    if {$::tcl_platform(platform) == "unix"} {
        set cursorspec "xterm red black"
    } else {
        set cursorspec "xterm"
    }
    entry $w.mesg.e \
        -font fnt-tiny \
        -highlightbackground black \
        -background "$::darkBackground" \
        -insertbackground "$::contrastForeground" \
        -textvariable "RTCChat::sendText" \
        -cursor $cursorspec
    button $w.mesg.b \
        -text "tx"\
        -font fnt-small\
        -highlightbackground black \
        -background "$::darkBackground" \
        -pady 0 \
        -padx 0\
        -takefocus 0 \
        -activebackground "$::darkBackground" \
        -activeforeground "$::contrastForeground" \
        -command "RTCChat::Send"
    bind $w.mesg.e <Return> "$w.mesg.b invoke"

    grid $w.mesg.e -row 0 -col 0 -stick news
    grid [frame $w.mesg.f] -row 0 -col 1 -padx 2
    grid $w.mesg.b -row 0 -col 2 
    grid columnconfigure $w.mesg 0 -weight 1

    grid $w.text -row 0 -col 0 -stick news
    grid [frame $w.f] -row 1 -col 0 -pady 1
    grid $w.mesg -row 2 -col 0 -stick ew
    grid columnconfigure $w 0 -weight 1
    grid rowconfigure    $w 0 -weight 1
   
    return $w
}


proc RTCChat::Send {} {
    if {$RTCChat::sendText != ""} {
        regsub -all "\n" $RTCChat::sendText " " RTCChat::sendText
        rtcsocket send "CHAT $RTCChat::sendText"
        set RTCChat::sendText ""
    }
}

proc RTCChat::AddText {message} {
    $RTCChat::text configure -state normal
    $RTCChat::text insert end "\n[join $message]"
    $RTCChat::text configure -state disabled
    $RTCChat::text see end
    return 1
}
