# Copyright 2001 Bryan Schofield
# This file is part of RoboTournament.
# 
# RoboTournament is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RoboTournament is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RoboTournament; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package provide RTCGameParam 1.0



proc ::rtcgameparam {subcommand args} {
    set help "Should be:\nrtcgameparam send"
    switch -- [string tolower $subcommand] {
        send { RTCGameParam::Construct }
        default {
            return -code error \
                "Invalid command rtcgameparam $subcommand $args\n$help"
        }
    }
}

namespace eval RTCGameParam {
    variable freq
    set freq(m1) 22.0
    set freq(m2) 10.0 
    set freq(m3)  5.0
    set freq(bu) 10.0
    set freq(jp)  5.0
    set freq(tr) 17.0
    set freq(tl) 17.0
    set freq(ut)  7.0
    set freq(no)  3.0
    set freq(w1)  1.0
    set freq(w2)  1.0
    set freq(w3)  1.0
    set freq(w4)  0.5
    set freq(w5)  0.5
    set total ""
    variable totalLabel
    variable default
    set default(m1) 22.0
    set default(m2) 10.0 
    set default(m3)  5.0
    set default(bu) 10.0
    set default(jp)  5.0
    set default(tr) 17.0
    set default(tl) 17.0
    set default(ut)  7.0
    set default(no)  3.0
    set default(w1)  1.0
    set default(w2)  1.0
    set default(w3)  1.0
    set default(w4)  0.5
    set default(w5)  0.5

    variable game 0
    variable gameRb
    variable mapPath ""
    variable mapDisplay ""

}

proc RTCGameParam::Construct {} {
    update
    set w ._game_param_
    destroy $w
    toplevel $w
    wm protocol $w WM_DELETE_WINDOW "bell"
    wm title $w "RoboTournament"
    wm iconname $w "RoboTournament"
    wm geometry $w 578x440+[expr [winfo screenwidth $w]/2-289]+[expr [winfo screenheight $w]/2-220]

    RTCGameParam::CardFreqController $w.cards
    #RTCGameParam::GameTypeController $w.game
    RTCGameParam::MapController $w.map

    button $w.ok \
        -text "OK" \
        -background "$::darkBackground" \
        -highlightthickness 1 \
        -highlightbackground "black" \
        -command "RTCGameParam::SendGameData $w" \
        -activebackground "$::darkForeground" \
        -borderwidth 0 \
        -relief flat \
        -width 16
    
    grid $w.cards -stick new  -padx 4 -pady 4 -ipady 2 -ipadx 2
    grid $w.map   -stick new  -padx 4 -pady 4
    grid $w.ok    -padx 4 -pady 4
    grid rowconfigure $w 2 -weight 1
    grab $w
    raise $w
    tkwait window $w
    grab release $w
}


proc RTCGameParam::MapController {f} {
    frame $f \
        -relief flat \
        -borderwidth 0 \
        -background "$::darkBackground" \
        -highlightthickness 1 \
        -highlightbackground "black"
    label $f.l \
        -text "Map" \
        -background "$::darkBackground" \
        -anchor e
    label $f.e \
        -textvariable RTCGameParam::mapDisplay \
        -background "$::background" \
        -highlightthickness 1 \
        -highlightbackground "black" \
        -anchor w
    button $f.s \
        -text "Select..." \
        -background "$::background" \
        -highlightthickness 1 \
        -highlightbackground "black" \
        -command "RTCGameParam::SelectMap $f" \
        -activebackground "$::darkForeground" \
        -borderwidth 0 \
        -relief flat \
        -padx 0 -pady 0
    grid $f.l -row 0 -col 0 -stick news -padx 4 -pady 4
    grid $f.e -row 0 -col 1 -stick news -pady 4
    grid $f.s -row 0 -col 2 -stick news -padx 4 -pady 4
    grid columnconfigure $f 1 -weight 1
    return $f
}


proc RTCGameParam::GameTypeController { f } {
    frame $f \
        -relief flat \
        -borderwidth 0 \
        -background "$::darkBackground" \
        -highlightthickness 1 \
        -highlightbackground "black"
    set col 0
    set row 0
    set i 0
    foreach type [list "Death Match" "Capture The Flag" "Domination" "Rally"] {
        set RTCGameParam::gameRb($i) \
            [radiobutton $f.g$i \
                 -text "$type" \
                 -variable RTCGameParam::game \
                 -value $i \
                 -indicatoron 0 \
                 -relief flat \
                 -borderwidth 0 \
                 -highlightthickness 1 \
                 -highlightbackground "black" \
                 -width 14 \
                 -selectcolor "$::background" \
                 -activebackground "$::darkForeground"]
        grid $f.g$i -col $col -row $row -stick ew -padx 2 -pady 2
        incr col
        if {$col > 1} {
            set col 0
            incr row
        }
        incr i
    }
    trace variable RTCGameParam::game w RTCGameParam::SetRbColor
    RTCGameParam::SetRbColor
    grid columnconfigure $f {0 1} -weight 1
    return $f
}

proc RTCGameParam::SetRbColor {args} {
    foreach name [array names RTCGameParam::gameRb] {
        if {$RTCGameParam::game == $name} {
            $RTCGameParam::gameRb($name) configure \
                -foreground "$::contrastForeground"
        } else {
            $RTCGameParam::gameRb($name) configure \
                -foreground "$::foreground"
        }
    }
}

proc RTCGameParam::CardFreqController { f } {
    frame $f \
        -relief flat \
        -borderwidth 0 \
        -background "$::darkBackground" \
        -highlightthickness 1 \
        -highlightbackground "black"
    set max [expr [llength [array names RTCGameParam::freq]]/3]
    set i 0
    set col 0
    set cards [list m1 m2 m3 jp bu tl tr ut no w1 w2 w3 w4 w5]
    foreach card $cards {
        set ff [frame $f.f_$card \
                    -background "$::background" \
                    -highlightthickness 1 \
                   -highlightbackground "black"]
        label $ff.i_$card \
            -image "img-$card"
        label $ff.l_$card \
            -textvariable "RTCGameParam::freq($card)" \
            -width 4 \
            -anchor se \
            -background "$::background"
        RTCGameParam::Scale $ff.s_$card RTCGameParam::freq($card) 0 30
        grid $ff.i_$card -row 0 -col 0 -rowspan 2
        grid $ff.l_$card -row 0 -col 1 -stick news
        grid $ff.s_$card -row 1 -col 1
        grid columnconfigure $ff {1 2} -weight 1
        grid $ff -col $col -row $i -padx 2 -pady 2 -ipadx 4 -ipady 4
        incr i
        if {$i >= $max} { incr col ; set i 0 }
        trace variable RTCGameParam::freq($card) w RTCGameParam::SumCardFreqs
    }

   set ff [frame $f.b \
             -relief flat \
             -borderwidth 0 \
             -highlightthickness 1 \
             -highlightbackground "black" \
             -background "$::background"]

    button $ff.restore \
        -text "Restore" \
        -relief flat \
        -borderwidth 0 \
        -highlightthickness 1 \
        -highlightbackground "black" \
        -background "$::darkBackground" \
        -activebackground "$::darkForeground" \
        -command "RTCGameParam::RestoreDefaultCardFreq" \
        -padx 0 -pady 0

    button $ff.normal \
        -text "Conform" \
        -relief flat \
        -borderwidth 0 \
        -highlightthickness 1 \
        -highlightbackground "black" \
        -background "$::darkBackground" \
        -activebackground "$::darkForeground" \
        -command "RTCGameParam::NormalizeCardFreq" \
        -padx 0 -pady 0
    grid $ff.restore -stick news -padx 4 -pady 4
    grid $ff.normal  -stick news -padx 4 -pady 4
    grid rowconfigure $ff {0 1} -weight 1
    grid columnconfigure $ff 0 -weight 1
        
    grid $f.b -col $col -row $i -padx 2 -pady 2 -ipadx 2 -ipady 2 -stick news
    incr i

    set ff [frame $f.f_total \
                -background "$::darkBackground"]
    label $ff.total \
        -text "Total" \
        -background "$::darkBackground"
    set RTCGameParam::totalLabel \
        [label $ff.var \
             -textvariable "RTCGameParam::total" \
             -width 4 \
             -anchor e \
             -background "$::darkBackground"]
    grid $ff.total -stick ew
    grid $ff.var   -stick ew
    grid $ff -col $col -row $i -padx 4



    RTCGameParam::SumCardFreqs
    return $f
}


proc RTCGameParam::RestoreDefaultCardFreq {} {
    foreach card [array names RTCGameParam::freq] {
        set RTCGameParam::freq($card) $RTCGameParam::default($card)
    }
}

proc RTCGameParam::NormalizeCardFreq {} {
   set diff [expr {100 - $RTCGameParam::total}]
   if {$diff == 0} { return }
   set numCards [llength [array names RTCGameParam::freq]]
   set amount [RTCGameParam::RoundAmount \
               [expr {$diff / double($numCards)}]]
   foreach name [array names RTCGameParam::freq] {
      set newAmount [expr {$RTCGameParam::freq($name) + $amount}]
      if {$newAmount >= 0 && $newAmount <= 30.0} {
         set RTCGameParam::freq($name) $newAmount
      }
         
   }
   set diff [expr {100 - $RTCGameParam::total}]
   if {$diff == 0} { return }
   if {$diff > 0} {
      set amount 0.5
   } else {
      set amount -0.5
   }
   set favors [list m1 tl tr m2 ut bu m3 jp]
   set i 0
   while {$RTCGameParam::total != 100} {
      set name [lindex $favors $i]
      set newAmount [expr {$RTCGameParam::freq($name) + $amount}]
      if {$newAmount >= 0 && $newAmount <= 30.0} {
         set RTCGameParam::freq($name) $newAmount
      }
      incr i
      if {$i >= [llength $favors]} {
         set i 0
      }
   }
}

proc RTCGameParam::SumCardFreqs {args} {
    set total 0
    foreach name [array names RTCGameParam::freq] {
        set total [expr $total +  $RTCGameParam::freq($name)]
    }
    if {$total != 100.0} {
        $RTCGameParam::totalLabel configure -fg "red"
    } else {
        $RTCGameParam::totalLabel configure -fg "$::foreground"
    }

    set RTCGameParam::total $total
}




proc RTCGameParam::Scale {w var min max} {
    canvas $w \
        -width 76 \
        -height 15 \
        -background "$::background" \
        -highlightthickness 1 \
        -highlightbackground "black" \
        -relief flat \
        -borderwidth 0 \
        -closeenough 2.0
    $w create line 1 1 1 15 \
        -width 1 \
        -fill "$::foreground" \
        -tag "tick"
    $w bind "tick" <ButtonPress-1> \
        "RTCGameParam::GrabTick $w $var $min $max"
    trace variable $var w "RTCGameParam::SetScale $w $var $min $max"
    RTCGameParam::SetScale $w $var $min $max
    return $w
}

proc RTCGameParam::GrabTick {w var min max} {
    $w bind "tick" <ButtonRelease-1> \
        "RTCGameParam::DropTick $w"
    $w bind "tick" <Motion> \
        "RTCGameParam::DragTick $w $var $min $max %x"
    $w itemconfigure "tick" -fill "$::contrastForeground"
}

proc RTCGameParam::RoundAmount { value } {
    set intVal [expr int($value)]
    set decVal [expr $value - $intVal]
    if {$decVal < 0.3} { 
        set decVal 0
    } elseif {$decVal < 0.8} {
        set decVal 0.5
    } else {
        set decVal 1.0
    }
   return [expr double($intVal + $decVal)]
}

proc RTCGameParam::DragTick {w var min max x} {
    if {($x > 75) || ($x < 0)} { return }
    set value [expr ($x/75.0) * ($max - $min) + $min]
    set x [expr $x + 1]
    #$w coord "tick" $x 0 $x 15
    set $var [RTCGameParam::RoundAmount $value]
}

proc RTCGameParam::DropTick {w} {
    $w bind "tick" <ButtonRelease-1> ""
    $w bind "tick" <Motion> ""
    $w itemconfigure "tick" -fill "$::foreground"    
}

proc RTCGameParam::SetScale {w var min max args} {
    eval set value $$var
    set x [expr ($value - $min) * 75.0/($max + $min) + 1]
    $w coord "tick" $x 0 $x 15
}


proc RTCGameParam::SelectMap { parent } {
    set filetypes {
            {"RoboTournament Map" {.map}} 
            {"All Files" {*}}
    }
    option add *Canvas.background #6e6e42
    set map \
        [tk_getOpenFile \
             -title "RoboTournament: Select Map" \
             -parent $parent \
             -filetypes $filetypes \
             -initialdir $::mapDir]
    option add *Canvas.background $::background
    if {$map == ""} { return }

    set RTCGameParam::mapPath $map
    set RTCGameParam::mapDisplay [file rootname [file tail $map]]
}


proc RTCGameParam::SendGameData { w } {
    if {$RTCGameParam::mapPath == ""} { bell ; return }
    if {$RTCGameParam::total != 100.0} { bell ; return }
    set mapType [lindex [split [file tail $RTCGameParam::mapPath] -] 0]
    switch -- [string tolower $mapType] {
        dm  { }
        ctf { }
        dom { }
        rly { }
        default {
            tk_messageBox \
                -parent $w \
                -title "RoboTournament Error" \
                -icon error \
                -message "Error: Unknown map type: $mapType\nSelect another map"
            return
        }

    }

    set data "MAP "
    append data "$mapType "
    set cards [list m1 m2 m3 jp bu tl tr ut no w1 w2 w3 w4 w5]
    foreach card $cards {
        append data "$RTCGameParam::freq($card) "
    }


    set f [open $RTCGameParam::mapPath r]
    while {[gets $f line] >= 0} {
        if {([llength $line] == 3) 
            && ([string first "\#" $line ] == -1)} {
            append data "$line "
        }
    }

    close $f
    rtcsocket send "$data"
    destroy $w
}
