# Copyright 2001 Bryan Schofield
# This file is part of RoboTournament.
# 
# RoboTournament is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RoboTournament is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RoboTournament; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package provide RTCGui 1.0
package require RTCRegister
package require RTCCardStorage
package require RTCChat
package require RTCBotStatus
package require RTCArena
package require RTCServerMsg

proc ::rtcgui {subcommand args} {
    switch -- [string tolower $subcommand] {
        construct {
            return [RTCGui::Construct]
        }
        drawarena {
            return [rtcarena draw $args]
        }
        startcontroller {
            return [rtccardstorage startcontroller]
        }
        powerupcontroller {
            return [rtccardstorage powerupcontroller]
        }
        turnstat {
            return [rtccardstorage turnstat $args]
        }
    }
    return -code error "Invalid arguments: Should be:\n\trtcgui construct"
}


namespace eval RTCGui {
    variable register ""
    variable cardstorage ""
    variable chat ""
    variable botstatus ""
    variable arena ""
    variable servermsg ""
}



proc RTCGui::Construct {} {
    set w .__root
    if {$::tcl_platform(platform) == "unix"} {
        set cursorspec "tcross red black"
    } else {
        set cursorspec "tcross"
    }
    toplevel $w \
        -cursor $cursorspec
    wm protocol $w "WM_DELETE_WINDOW" "RTC::Exit"
    wm title $w "RoboTournament"
    wm iconname $w "RoboTournament"
    wm geometry $w 800x600+[expr [winfo screenwidth $w]/2-400]+[expr [winfo screenheight $w]/2-300]
    
    bind $w <Escape> "RTC::Exit"
    bind $w <Control-Escape> "RTC::Exit force"

    set f [frame $w.cards -bd 0]    
    set RTCGui::register [rtcregister construct $f.regs]
    set RTCGui::cardstorage [rtccardstorage construct $f.stor]
    set RTCGui::chat [rtcchat construct $f.chat]
    set RTCGui::botstatus [rtcbotstatus construct $f.bots]
    set RTCGui::arena [rtcarena construct $w.arena]
    set RTCGui::servermsg [rtcservermsg construct $w.serv]

    grid $RTCGui::botstatus   -stick ew -padx 0  -pady 0
    grid $RTCGui::chat        -stick news -padx 0 -pady 4
    grid $RTCGui::register    -padx 0 -pady 0 
    grid $RTCGui::cardstorage -padx 0 -pady 0
    grid rowconfigure $f 1 -weight 1
    grid columnconfigure $f 0 -weight 1

    
    grid $w.arena -row 0 -col 0 -stick news -padx 4 -pady 4
    grid $w.serv  -row 1 -col 0 -stick ew   -padx 4 -pady 4
    grid $w.cards -row 0 -col 1 -stick news -padx 4 -pady 4 -rowspan 2
    grid rowconfigure $w 0 -weight 1
    grid columnconfigure $w 0 -weight 1
}


