# Copyright 2001 Bryan Schofield
# This file is part of RoboTournament.
# 
# RoboTournament is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RoboTournament is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RoboTournament; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package provide RTCLogin 1.0
package require RTCSocket

namespace eval RTCLogin {
    variable data
    set data(Host) "localhost"
    set data(Port) "4390"
    set data(Name) "CheeseBot"
    set data(Bot) 0

    variable flashCycle 0
    variable tryLogin 0

}

proc ::rtclogin {} {
    return [RTCLogin::MakeLoginGui]
}

proc RTCLogin::MakeLoginGui {} {



    set w .__login
    destroy $w
    toplevel $w
    wm protocol $w "WM_DELETE_WINDOW" { set ::RTCLogin::tryLogin CANCEL }
    wm title $w "RoboTournament"
    wm iconname $w "RoboTournament"
    wm geometry $w 500x350+[expr [winfo screenwidth $w]/2-250]+[expr [winfo screenheight $w]/2-175]
    #
    # The data
    #
    set f [frame $w.data]
    foreach thing [list Host Port Name] {
        label $f.l_$thing \
            -text "$thing" 
        entry $f.e_$thing \
            -textvariable "RTCLogin::data($thing)" \
            -highlightbackground "black" \
            -highlightthickness 1 \
            -highlightcolor "black" \
            -background "$::darkBackground" \
            -relief flat \
            -insertbackground $::contrastForeground
        grid $f.l_$thing $f.e_$thing -stick ew -padx 6 -pady 6
    }
    grid columnconfigure $f 1 -weight 1
    focus $f.e_Host


    set f [frame $w.bots]
    for { set i 0 } {$i < 8} { incr i } {
        radiobutton $f.b$i \
            -image "img-bot-$i-n" \
            -value $i \
            -variable "RTCLogin::data(Bot)" \
            -relief flat \
            -indicatoron 0 \
            -borderwidth 0 \
            -selectcolor "$::darkForeground" \
            -highlightthickness 1 \
            -highlightbackground "black" \
            -highlightcolor "black" \
            -background "$::darkBackground" \
            -padx 4 \
            -pady 4
        grid $f.b$i -row 0 -col $i -padx 2
    }



    set f [frame $w.buts]
    button $f.cancel \
        -text "Abort" \
        -width 10 \
        -highlightbackground black \
        -highlightthickness 1 \
        -highlightcolor "black" \
        -background "$::darkBackground" \
        -command { set ::RTCLogin::tryLogin CANCEL } \
        -activebackground "$::darkForeground"\
        -takefocus 0 \
        -borderwidth 1
    button $f.ok \
        -text "Engage" \
        -width 10 \
        -highlightbackground black \
        -highlightthickness 1 \
        -highlightcolor "black" \
        -background "$::darkBackground"\
        -command { set ::RTCLogin::tryLogin GO } \
        -activebackground "$::darkForeground"\
        -takefocus 0 \
        -borderwidth 1
    button $f.start \
        -text "Start Server" \
        -highlightbackground black \
        -highlightthickness 1 \
        -highlightcolor "black" \
        -background "$::darkBackground"\
        -command { RTCLogin::StartServer } \
        -activebackground "$::darkForeground"\
        -takefocus 0\
        -borderwidth 1
    RTCLogin::FlashButton $f.ok
    grid $f.ok     -row 0 -col 0 -stick w -padx 6 -pady 6
    grid $f.cancel -row 0 -col 1 -stick e -padx 6 -pady 6
    grid $f.start  -row 1 -col 0 -stick ew -padx 6 -pady 6 -columnspan 2
    grid columnconfigure $f {0 1} -weight 1
        
    
    grid $w.data -stick news -padx 6 -pady 6
    grid $w.bots -stick ew   -padx 6 -pady 6
    grid $w.buts -stick ew   -padx 6 -pady 6
    grid columnconfigure $w 0 -weight 1
    grid rowconfigure    $w 1 -weight 1


    bind $w <Return> { set ::RTCLogin::tryLogin GO }
    bind $w <Escape> { set ::RTCLogin::tryLogin CANCEL }
    

    while {$RTCLogin::tryLogin != "CANCEL"} {
        vwait ::RTCLogin::tryLogin
        if {$RTCLogin::tryLogin == "CANCEL"} {
            destroy $w
            return 0 
        }
        if {[llength $RTCLogin::data(Host)] == 0} {
            tk_messageBox \
                -title "RoboTournament" \
                -icon "warning" \
                -message "Invalid host name."
            continue
        }
        if {([llength $RTCLogin::data(Name)] == 0)
            || ([string is alnum $RTCLogin::data(Name)] == 0)} {
            tk_messageBox \
                -title "RoboTournament" \
                -icon "warning" \
                -message "Invalid name."
            continue
        }
        if {([llength $RTCLogin::data(Port)] == 0)
            || ([string is integer $RTCLogin::data(Port)] == 0)} {
            tk_messageBox \
                -title "RoboTournament" \
                -icon "warning" \
                -message "Invalid port."
            continue
        }
        set status [rtcsocket open $RTCLogin::data(Host) $RTCLogin::data(Port)]
        if { $status == 1} {
            rtcsocket send \
                "REGISTER $RTCLogin::data(Name) $RTCLogin::data(Bot)"
            destroy $w
            return 1
        } else {
            tk_messageBox \
                -title "RoboTournament" \
                -icon "error" \
                -message "Failed to connect to $RTCLogin::data(Host) on port $RTCLogin::data(Port)\n$status"
        }
    }
    destroy $w
    return 0
}

proc RTCLogin::StartServer {} {
    if {[catch {exec [info nameofexecutable] [file join $::homeDir .. server RTServer.tcl] &} err]} {
        tk_messageBox \
		-icon error \
		-title "RoboTournamanent" \
		-message "Unable to start server: $err"
    }
}


proc RTCLogin::FlashButton {widget} {
    if {![winfo exists $widget]} {return}
    if {$RTCLogin::flashCycle == 1} {
        set RTCLogin::flashCycle 0
        $widget configure \
            -foreground $::foreground
    } else {
        set RTCLogin::flashCycle 1
        $widget configure \
            -foreground $::contrastForeground 
    }
    after 1000 "RTCLogin::FlashButton $widget"
}
