# Copyright 2001 Bryan Schofield
# This file is part of RoboTournament.
# 
# RoboTournament is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RoboTournament is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RoboTournament; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package provide RTCRegister 1.0


proc ::rtcregister {subcommand args} {
    set help "\nShould be:\n\trtcregister construct <path>\n\trtcregister setcard <regnum> <card>\n\trtcregister getcard <regnum>\n\trtcregister removecard <regnum>" 
    #
    # subcommands that do not require register number
    #
    switch -- [string tolower $subcommand] {
        "construct" {
            return [RTCRegister::Construct $args]
        }
        "addcard" {
            return [RTCRegister::AddCard $args]
        }
        "setweapons" {
            return [RTCRegister::SetWeapons [join $args]]
        }
        "enable" {
            $RTCRegister::uploadbutton configure -state normal
            $RTCRegister::powerbutton configure -state normal
            return 1
        }
    }

    #
    # subcommands that do require register number
    #
    if {([string is integer [lindex $args 0]] == 0)
        || ([lindex $args 0] < 0) 
        || ([lindex $args 0] > 4)} {
        return -code error \
            "Invalid register number: [lindex $args 0] $help"
    }

    switch -- [string tolower $subcommand] {
        setcard {
            if {[llength $args] != 2} { 
                return -code error \
                    "Not enough arguments: $subcommand $args $help"
            }
            return [RTCRegister::SetCard [lindex $args 0] [lindex $args 1]]
        }
        getcard {
            return [RTCRegister::GetCard [lindex $args 0]]
        }
        removecard {
            return [RTCRegister::RemoveCard [lindex $args 0]]
        }
        lock {
            return [RTCRegister::LockCard [lindex $args 0]]
        }
        unlock {
            return [RTCRegister::UnLockCard [lindex $args 0]]
        }
        islocked {
            return $RTCRegister::locked([lindex $args 0])
        }
    }
    return -code error "Unknown command: \"$subcommand\" \"$args\" $help"
}


namespace eval RTCRegister {
    variable canvas
    variable x
    variable y
    variable card
    variable weapon 0
    variable weaponbutton
    variable uploadbutton
    variable powerbutton
    variable power 1
    variable locked
}


proc RTCRegister::Construct { w } {
    
    frame $w

    set RTCRegister::canvas [canvas $w.c \
                                 -width 285 \
                                 -height 85\
                                 -borderwidth 0 \
                                 -highlightthickness 0]


    $RTCRegister::canvas create image 0 0 \
        -anchor nw \
        -image img-register-bg \
        -tag bg

    set x 10
    set y 10
    for { set i 0} {$i < 5} {incr i} {
        set RTCRegister::x($i) $x
        set RTCRegister::y($i) $y
        incr x 55
        set RTCRegister::card($i) ""
        set RTCRegister::locked($i) 0
    }


    set f [frame $w.f \
              -relief flat \
               -borderwidth 0 \
               -background "$::darkBackground" \
               -highlightthickness 1 \
               -highlightbackground "black"]
    
    for {set i 0} {$i < 6} {incr i} {
        radiobutton $f.w$i \
            -image img-weapon-$i \
            -selectimage img-weapon-$i-select\
            -borderwidth 0 \
            -highlightthickness 1 \
            -highlightbackground "black" \
            -indicatoron 0 \
            -variable "RTCRegister::weapon" \
            -value $i \
            -state normal \
            -takefocus 0
        set RTCRegister::weaponbutton($i) $f.w$i
        grid $f.w$i -row 0 -col $i -padx 2 -pady 4 -stick w
    }
    $f.w0 configure -state normal

    set RTCRegister::uploadbutton \
        [button $w.up \
             -relief flat \
             -background "$::darkBackground" \
             -highlightthickness 1 \
             -highlightbackground "black" \
             -activebackground "$::darkBackground" \
             -borderwidth 0 \
             -image "img-upload" \
             -font fnt-tiny \
             -anchor nw \
             -takefocus 0 \
             -activeforeground "$::contrastForeground" \
             -command "RTCRegister::UpLoadProgram " \
             -state disabled]

    set RTCRegister::powerbutton \
        [checkbutton $w.pow \
             -relief flat \
             -background "$::darkBackground" \
             -highlightthickness 1 \
             -highlightbackground "black" \
             -borderwidth 0 \
             -indicatoron 0 \
             -variable "RTCRegister::power" \
             -image "img-power-off" \
             -selectimage "img-power-on"\
             -takefocus 0]

    grid $w.f   -row 0 -col 0 -stick w -ipadx 2
    grid $w.pow -row 0 -col 1  -padx 4
    grid $w.up  -row 0 -col 2  -stick e
    grid [frame $w.s -height 4] -row 1 -col 0 
    grid $w.c  -row 2 -col 0 -stick news -columnspan 3
    grid columnconfigure $w {1} -weight 1

    return $w
}



#
# Card Procs
#

proc RTCRegister::AddCard { card } {
    for { set i 0 } { $i < 5 } { incr i} {
        if { $RTCRegister::card($i) == "" 
             && ($RTCRegister::locked($i) == 0)} {
            RTCRegister::SetCard $i $card
            return 1
        }
    }
    return 0
}

proc RTCRegister::SetCard {regnum card} {
    RTCRegister::RemoveCard $regnum
    if {![regexp {^img-} $card]} {
        set RTCRegister::card($regnum) "$card"
        set card img-$card
    } else {
        set RTCRegister::card($regnum) [string range 4 end]
    }
    $RTCRegister::canvas create image \
        $RTCRegister::x($regnum) $RTCRegister::y($regnum) \
        -image $card \
        -anchor nw \
        -tag "card-$regnum"
    $RTCRegister::canvas bind "card-$regnum" <1> \
        "RTCRegister::ClickCard $regnum"
    return 1
}


proc RTCRegister::RemoveCard {regnum} {
    $RTCRegister::canvas delete "card-$regnum"
    set RTCRegister::card($regnum) ""
    return 1
}

proc RTCRegister::GetCard {regnum} {
    return $RTCRegister::card($regnum)
}

proc RTCRegister::ClickCard {regnum} {
    if {$RTCRegister::locked($regnum)} { return }
    if {[rtccardstorage addcard [RTCRegister::GetCard $regnum]]} {
        RTCRegister::RemoveCard $regnum
    }
}

proc RTCRegister::LockCard {regnum} {
    set RTCRegister::locked($regnum) 1
    $RTCRegister::canvas delete "lock-$regnum"
    $RTCRegister::canvas create image \
        $RTCRegister::x($regnum) $RTCRegister::y($regnum) \
        -image "img-lock-overlay" \
        -anchor nw \
        -tag "lock-$regnum"

}

proc RTCRegister::UnLockCard {regnum} {
    set RTCRegister::locked($regnum) 0
    $RTCRegister::canvas delete "lock-$regnum"
}


proc RTCRegister::SetWeapons {weapons} {
    #puts "Available Weapons: $weapons"
    set RTCRegister::weapon 0
    set RTCRegister::power 1
    for {set i 0} {$i < 6} {incr i} {
        $RTCRegister::weaponbutton($i) configure -state disabled
    }
    foreach weapon $weapons {
        $RTCRegister::weaponbutton($weapon) configure -state normal
    }
}

proc RTCRegister::UpLoadProgram {} {
    set packet "PROGRAM $RTCRegister::power $RTCRegister::weapon"
    for { set i 0 } { $i < 5 } { incr i} {
        if { $RTCRegister::card($i) == "" } { return }
        set packet "$packet $RTCRegister::card($i)"
    }
    rtcsocket send $packet
    $RTCRegister::uploadbutton configure -state disabled
    $RTCRegister::powerbutton configure -state disabled
    for { set i 0 } { $i < 9 } { incr i } {
        rtccardstorage removecard $i
    }
    for { set i 0 } { $i < 5 } { incr i} {
        rtcregister lock $i
    }
    for {set i 0} {$i < 6} {incr i} {
        $RTCRegister::weaponbutton($i) configure -state disabled
    }
    
}
