# Copyright 2001 Bryan Schofield
# This file is part of RoboTournament.
# 
# RoboTournament is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RoboTournament is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RoboTournament; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package provide RTCSocket 1.0

#
# Provides:
# rtcsocket open <host> <port>
# rtcsocket send <message>
# rtcsocket close
#

proc ::rtcsocket { subcommand args } {
    switch -- [string tolower $subcommand] {
        open {
            if {[llength $args] != 2} {
                return -code error \
                    "Invalid arguments: rtcsocket open <host> <port>"
            } else {
                return [RTCSocket::Connect [lindex $args 0] [lindex $args 1]]
            }
        }
        close {
            return [RTCSocket::Disconnect]
        }
        send {
            if {[llength $args] != 1} {
                 return -code error \
                     "Invalid arguments: rtcsocket send <message>"
            } else {
                return [RTCSocket::Send $args]
            }
        }
        default {
            return -code error \
                "Invalid arguments: Should be:\n\trtcsocket open <host> <port>\n\trtcsocket close\n\trtcsocket send <message>"
        }
    }
}

namespace eval RTCSocket {
    variable connected 0
    variable debug 0
    variable server
}

#========================================================================
proc RTCSocket::Dputs {msg} {
    if {$RTCSocket::debug} {
        puts stderr "RTCSocket DEBUG> $msg"
    }
}

#========================================================================
proc RTCSocket::Connect {host port} {
    if {$RTCSocket::connected} {
        RTCSocket::Dputs "Already connected to $host on $port"
        return 1
    }
    RTCSocket::Dputs "Trying to connect to $host on port $port"
    if {[catch {socket -async $host $port} channel]} {
        RTCSocket::Dputs "Could not connect. $channel"
        return $channel
    }
    set RTCSocket::server $channel
    fileevent $RTCSocket::server readable RTCSocket::Receive
    set RTCSocket::connected 1
    return 1
}


#========================================================================
proc RTCSocket::Receive {} {
    if {[eof $RTCSocket::server]} {
        RTCSocket::Dputs "Server Connection Closed"
        close $RTCSocket::server
        set RTCSocket::connected 0
        tk_messageBox \
            -icon error \
            -title "Network Error" \
            -message "Connection to server has been terminated"
        exit
    }
    if {[catch {gets $RTCSocket::server message}]} {
        puts "\nSorry...Not connected to $::host on port $::port\nMake sure there is an active server.\n"
        Usage
    }
    RTCSocket::Dputs "RTCSocket Recieved: $message"
    if {$message == "UPDATE_IDLE_TASKS"} {
        RTCSocket::UpdateIdletasks
    } else {
        RTC::MessageHandler $message
    }
}

#========================================================================
proc RTCSocket::Send {message} {
    RTCSocket::Dputs "RTCSocket Sending: $message"
    if {!$RTCSocket::connected} { return }
    if {[catch {puts $RTCSocket::server $message } err]} {
        RTCSocket::Dputs "RTCSocket Send Error: $err"
	  return
    }
    if {[catch {flush $RTCSocket::server}]} {
	  tk_messageBox \
		-icon error \
		-title "RoboTournament Error" \
		-message "Error communicating with server socket:\n$err\nExiting."
	  RTC::Exit "force"
    }
}

#========================================================================
proc RTCSocket::UpdateIdletasks {} {
    update idletasks
    RTCSocket::Send "I_AM_UPDATED"
}


#========================================================================
proc RTCSocket::Disconnect {} {
    set RTCSocket::connected 0
    close $RTCSocket::server
}
