# Copyright 2001 Bryan Schofield
# This file is part of RoboTournament.
# 
# RoboTournament is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RoboTournament is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RoboTournament; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package provide RTSBot 1.0
package require RTSBotAI 


#========================================================================
# rtsbot
#
# Public level access to robots
#
proc ::rtsbot {subcommand args} {
    set help "\nShould be:\n"
    append help "rtsbot bots\n"
    append help "rtsbot init <number>\n"
    append help "rtsbot isvalid <id>\n"
    append help "rtsbot sendnames\n"
    append help "rtsbot sendupdate\n"
    append help "rtsbot updateweapons\n"

    switch -- $subcommand {
        cardfrequency {
            set RTSBot::freq([lindex $args 0]) [lindex $args 1]
        }
        bots { 
            return $RTSBot::bots 
        }
        init { 
            RTSBot::Init $args 
        }
        isvalid { 
            return [info exists RTSBot::bot($args)] 
        }
        isclone {
            regsub "^bot" $args "" botId
            return [info exists RTSBot::bot($botId,clone)] 
        }
       sendnames {
            set names "NAMES "
            foreach bot $RTSBot::bots {
                append names "$bot \"[$bot name]\" "
            }
            rtssocket broadcast "$names"
        }
        sendupdate {
            set packet "BOTUPDATE $RTS::game(turn) $RTS::game(phase) "
            foreach bot $RTSBot::bots {
                if {[$bot placed]} {
                    append packet "$bot [$bot x] [$bot y] "
                    append packet "[$bot face] [$bot health] [$bot frags] "
                }
            }
            rtssocket broadcast "$packet"
        }
        updateweapons {
            foreach bot $RTSBot::bots {
                $bot updateweapons
            }
        }
        default {
            return -code error "Unknown command rtsbot $subcommand $args $help"
        }
    }
}



namespace eval RTSBot {
    variable bots
    variable bot

    variable name
    set name(0) "Tank"
    set name(1) "Goblin"
    set name(2) "GrassHopper"
    set name(3) "WhichWay"
    set name(4) "Lefty"
    set name(5) "Boxer"
    set name(6) "SpeedRacer"
    set name(7) "BigWheel"

    variable weaponName
    set weaponName(0) "single laser"
    set weaponName(1) "the Doubler"
    set weaponName(2) "the Tail Canon"
    set weaponName(3) "the Side Shooters"
    set weaponName(4) "the Four Way"
    set weaponName(5) "the BFG"

    #
    # Card frequencies
    #
    variable freq
    set freq(m1) 20
    set freq(m2)  8 
    set freq(m3)  5
    set freq(bu)  4
    set freq(jp)  5
    set freq(tr) 20
    set freq(tl) 20
    set freq(ut)  7
    set freq(no)  2
    set freq(w1)  2
    set freq(w2)  2
    set freq(w3)  2
    set freq(w4)  1.5
    set freq(w5)  1.5
}


#========================================================================
# RTSBot::Init <num>
#
# Constructs <num> number of bots
#
proc RTSBot::Init { num } {
    for { set i 0 } { $i < $num } { incr i } {
        lappend RTSBot::bots [RTSBot::Construct $i]
    }
}


#========================================================================
# RTSBot::Construct <id>
#
# Constructs a bot with id <id>
# Once a bot is constructed, a new command is also created:
#
#  bot<id>
#
# All attributes can be accessed with:
#
#  bot<id> <attribute>
#
# And all attributes can be modified with:
#
#  bot<id> <attribute> <value>
#
# NOTE: THERE IS NO VALIDATION DONE TO ATTRIBUTE VALUES
#
# Other subcommands are defined by the WidgetProc
#
proc RTSBot::Construct { id } {
    variable bot
    if {[info exists bot($id)]} {
        return -code error "Trying to re-construct bot $id. Do not do that."
    }
    set bot($id) 1
    if {[info exists RTSBot::name($id)]} {
        set bot($id,name) $RTSBot::name($id)
    } else {
        set bot($id,name) "Robot $id"
    }
    set bot($id,x) ""
    set bot($id,y) ""
    set bot($id,face) "n"
    set bot($id,weapon) 0
    set bot($id,availweapons) 0
    set bot($id,frags) 0.00
    set bot($id,health) 100
    set bot($id,program) ""
    set bot($id,cards) ""
    set bot($id,free) 1
    set bot($id,client) ""
    set bot($id,placed) 0
    set bot($id,powereddown) 0
    set bot($id,poweronnext) 1
    set bot($id,nextflag) 0
    set bot($id,team) ""

    proc ::bot$id { command args } \
        "namespace eval RTSBot Widgetproc $id \$command \$args"

    return bot$id
}
#========================================================================
# RTSBot::Widgetproc <id> <command> <args>
# 
# Implements the subcommands for the bot widgets
#
proc RTSBot::Widgetproc { id command args } {
    set help "Invalid command: bot$id $command $args\nShould be:\n"
    append help "bot$id addfrag <float>\n"
    append help "bot$id addweapon <weapon_id>\n"
    append help "bot$id dealcards\n"
    append help "bot$id die\n"
    append help "bot$id takedamage <damage>\n"
    append help "bot$id powerup <boolean>\n"
    append help "bot$id updateweapons\n"
    append help "bot$id safesend <message>\n"
    switch -- $command {
        addfrag       { RTSBot::AddFrag $id $args }
        addweapon     { RTSBot::AddWeapon $id $args }
        attributes    { return [list name x y face weapon availweapons \
                                    frags health program cards free \
                                    client placed powereddown poweronnext]}
        chooseprogram { RTSBotAI::ChooseProgram bot$id }
        clone         { return [RTSBot::Clone $id $args]}
        dealcards     { RTSBot::DealCards $id }
        destruct      { RTSBot::Destruct $id }
        die           { RTSBot::Die $id }
        cardindex     { return [RTSBot::CardIndex $id $args]}
        pickcard      { RTSBotAI::PickCard bot$id }
        programncards { RTSBotAI::ProgramNCards bot$id \
                            [lindex $args 0] [lindex $args 1]}
        powerup       { RTSBot::PowerUp $id $args }
        safesend      { RTSBot::SafeSend $id "[join $args]" }
        takedamage    { return [RTSBot::TakeDamage $id $args]}
        updateweapons { RTSBot::UpdateWeapons $id }
        default {
            if {[info exists RTSBot::bot($id,$command)]} {
                if {$args != ""} {
                  #
                  # Let's do some trimming to make sure there is no
                  # garbage on the value.
                  #
                  set RTSBot::bot($id,$command) \
                      [string trim [join $args] "\{\}\\ "]
                }
                return $RTSBot::bot($id,$command) 
            }
            return -code error "$help"
        }
    }
}


proc RTSBot::Destruct { id } {
    variable bot
    RTS::RemoveBotFromMap "bot$id"
    unset RTS::item(bot$id)
    unset bot($id)
    unset bot($id,name)
    unset bot($id,x)
    unset bot($id,y)
    unset bot($id,face)
    unset bot($id,weapon)
    unset bot($id,availweapons)
    unset bot($id,frags)
    unset bot($id,health)
    unset bot($id,program)
    unset bot($id,cards)
    unset bot($id,free)
    unset bot($id,client)
    unset bot($id,placed)
    unset bot($id,powereddown)
    unset bot($id,poweronnext)
    unset bot($id,nextflag)
    if {[info exists bot($id,clone)]} {
        unset bot($id,clone) 
    }
    rename "::bot$id" ""
}

proc RTSBot::Clone { id cid } {
    variable bot
    if {![info exists bot($id)]} {
        return -code error "Trying to clone non-existant bot $id. Do not do that."
    }
    set clone [RTSBot::Construct "$cid"]
    foreach att [bot$id attributes] {
        set bot($cid,$att) $bot($id,$att)
    }
    set bot($cid,clone) 1
    set RTS::item($clone) b
    return $clone
}



#========================================================================
# RTSBot::PowerUp <id> <powerup>
#
# Powers up and deals cards or leaves a powered down bot powered down
#
proc RTSBot::PowerUp { id powerup} {
    variable bot
    if {$powerup} {
        set bot($id,powereddown) 0
        puts "Powering Up bot$id"
        RTSBot::DealCards $id
    } else {
        RTS::ProcessProgram "bot$id" "1 0 x x x x x"
        #puts "Powering Down Continued bot$id"
    }
}

#========================================================================
proc RTSBot::CardIndex {id card} {
    variable bot
    set index [lsearch [split [join $bot($id,cards)]] $card]
    return $index
}

#========================================================================
proc RTSBot::DealCards { id } {
    variable bot
    variable freq
    if {$bot($id,powereddown) && !$bot($id,free)} {
        #puts "WANTTOPOWERUP $id?"
        RTSBot::SafeSend $id "WANTTOPOWERUP"
        return
    }
    if {!$bot($id,poweronnext)} {
        set bot($id,powereddown) 1
        RTS::ProcessProgram "bot$id" "1 0 x x x x x"
        #puts "Powering Down bot$id"
        return
    } else {
        set bot($id,powereddown) 0
    }
    set bot($id,program) ""
    set numCards [expr {round($bot($id,health) / 10.0) -1}]
    if {$numCards > 9} { set numCards 9 }
    if {($numCards < 5) && ($bot($id,free) == 1)} { set numCards 5 }
    set cards ""
    for { set i 0 } {$i < $numCards} {incr i} {
        set rand [expr {int(rand() * 100)}]
        set min 0
        foreach cardType [array names freq] {
            if {($rand >= $min) && ($rand < [expr {$min + $freq($cardType)}])} {
                lappend cards $cardType
                break
            } else {
                set min [expr {$min + $freq($cardType)}]
            }
        }
    }
    if {[llength $cards] < $numCards} {
        puts "RTSBot: Failed to deal all cards for bot $id\n\tDealt $cards\n\tShould have dealt $numCards\n\tHealth $bot($id,health)"
    }
    RTSBot::SafeSend $id "CARDS $cards"
    set bot($id,cards) $cards
}

#========================================================================
proc RTSBot::UpdateWeapons { id } {
    variable bot
    set weapons ""
    set removed 0
    foreach weapon $bot($id,availweapons) {
        if {($weapon == 0) || ($bot($id,weapon) != $weapon) || $removed} {
            set weapons "$weapons$weapon "
        } else {
            set removed 1
        }
    }
    set bot($id,availweapons) [split [join [string trim $weapons]]]
    #
    # Uncomment to allow only 1 usage to be stored
    #
    #set bot($id,availweapons) [lsort -unique $bot($id,availweapons)]
    RTSBot::SafeSend $id "WEAPONS $weapons"
    #puts "WEAPONS $id $weapons"
}

#========================================================================
proc RTSBot::SafeSend { id message } {
    variable bot
    if {!$bot($id,free) && ($bot($id,client) != "")} {
        rtssocket send $bot($id,client) "$message"
    }
}

#========================================================================
proc RTSBot::Die { id {fromlaser 0}} {
    variable bot
    if {![info exists bot($id,clone)]} {
      set bot($id,health) 0
      rtsbot sendupdate
    }
    set bot($id,placed) 0
    set bot($id,poweronnext) 1
    set bot($id,health) [expr {80 + round(rand() * 10)}]
    if {($RTS::game(type) == "ctf") 
        && ([join $bot($id,nextflag)] != "")
        && ![info exists bot($id,clone)]} {
        if {$fromlaser == 0} {
            # Put the flag on it's base
            foreach baseId $RTS::map(bases) {
                if {$bot($id,nextflag) == $RTS::item($baseId,team)} {
                    set x $RTS::item($baseId,x)
                    set y $RTS::item($baseId,y)
                    break
                }
            }
        } else {
            # Drop the flag
            set x $bot($id,x)
            set y $bot($id,y)            
        }
        puts "BOTDIE $id"
        RTS::PlaceItem [lindex $RTS::map(flag,$bot($id,nextflag)) 0] $x $y
        rtssocket broadcast "SETFLAG bot$id empty"
        set bot($id,nextflag) ""
    }
    RTS::RemoveBotFromMap "bot$id"
    set bot($id,frags) [expr {$bot($id,frags) - 0.5}]
}

#========================================================================
proc RTSBot::AddFrag { id amount } {
    variable bot
    set bot($id,frags) [expr {$bot($id,frags) + $amount}]
    if {($RTS::game(type) == "dm" ) 
        && ($bot($id,frags) >= $RTS::game(dmfrags))} {
        RTS::GameOver \
            "$bot($id,name) has won the game with $bot($id,frags) frags"
    }
}

#========================================================================
proc RTSBot::TakeDamage { id amount } {
    variable bot
    if {$amount < 0} {
      incr bot($id,health) [expr {abs($amount)}]
    } else {
      incr bot($id,health) -$amount
    }
    #set bot($id,health) [expr {round($bot($id,health) - $amount)}]
    if {![info exists bot($id,clone)] && ($amount < 0)} {
        rtssocket broadcast "SERVERMSG $bot($id,name) repairs [expr {abs($amount)}] points of damage"
    }
    if {$bot($id,health) <= 0} {
        #puts "Kill bot$id - RTSBot::TakeDamage"
        RTSBot::Die $id fromlaser
        if {![info exists bot($id,clone)]} {
            rtssocket broadcast "BLOWNUP bot$id"
        }
        return 0
    }
    return 1
}

#========================================================================
proc RTSBot::AddWeapon {id w} {
    variable bot
    lappend bot($id,availweapons) $w
    if {![info exists bot($id,clone)] 
        && [info exists RTSBot::weaponName($w)]} {
        rtssocket broadcast "SERVERMSG $bot($id,name) gains $RTSBot::weaponName($w)"
    }
}
