# Copyright 2001 Bryan Schofield
# This file is part of RoboTournament.
# 
# RoboTournament is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RoboTournament is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RoboTournament; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package provide RTSBotAI 1.0

namespace eval RTSBotAI {
    variable kamakazi
    variable navpoint
}

proc RTSBotAI::ChooseProgram { bot } {
    variable kamakazi
    if {[$bot powereddown]} {
      $bot poweronnext 1
      $bot weapon 0
      $bot program "x x x x x"
      return
    } 
    if {[info exists kamakazi($bot)] && ($kamakazi($bot,turns) <= 0)} {
        unset kamakazi($bot)
        unset kamakazi($bot,turns)
    }
    if {[$bot health] < 60} { 
        set power 0
    } else {
        set power 1
    }
    set weapon [lindex [split [join [$bot availweapons]]] end]
    if {$weapon == ""} { set weapon 0 }
    
    set dynamicAI 0
    if {$dynamicAI} {
      set program "p p p p p"
      RTS::ProcessProgram $bot \
         "$power $weapon $program"

    } else {
       $bot program "p p p p p"
       $bot weapon $weapon
       $bot poweronnext $power
       #set t [clock seconds]
       RTSBotAI::ProgramNCards $bot [expr {int(rand() * 4) + 2}] 0
       RTS::ProcessProgram $bot \
         "$power $weapon [$bot program]"
       #puts "[$bot name] took [expr [clock seconds] - $t] s to program [$bot program]"
    }
    #$bot poweronnext 1
}



proc RTSBotAI::ProgramNCards { bot numCards phase } {
   set x [$bot x]
   set y [$bot y]
   set dest [RTSBotAI::GetDestination $bot]
   set tx [lindex $dest 0]
   set ty [lindex $dest 1]
   set dx [expr {$tx - $x}]
   set dy [expr {$ty - $y}]
   set dt [expr {abs($dx) + abs($dy)}]
   if {$dx < 0} {
       set face(x) "w"
   } elseif {$dx > 0} { 
       set face(x) "e"
   } else {
       set face(x) "x"
   }
   if {$dy < 0} {
       set face(y) "n"
   } elseif {$dy > 0} {
       set face(y) "s"
   } else {
       set face(y) "x"
   }
   set face(bot) [$bot face]
   set face(90)  [RTS::GetRotatedDirection $face(bot) 90]
   set face(180) [RTS::GetRotatedDirection $face(bot) 180]
   set face(270) [RTS::GetRotatedDirection $face(bot) 270]

   set combinations ""
   for { set i 0 } {$i < [expr {10 * pow($numCards,2)}]} { incr i } {
      set cards [RTS::RandomizeList [split [$bot cards]]]
      set combination ""
      for { set j 0 } {$j < $numCards} { incr j } {
          set index $j
          lappend combination [lindex $cards $index]
      }
      lappend combinations $combination
   }

   set bestProgram ""
   set bestCombo ""
   set bestScore ""

   set origCombos [llength $combinations]
   set combinations [lsort -unique $combinations]

   #puts "[$bot name]: Placed? [$bot placed]: $origCombos -> [llength $combinations] : [$bot cards]"
   
   set count 0
   set sumTime 0
   set startTime [clock seconds]
   for {set loop 0} {$loop < [llength $combinations]} { incr loop } {
      set time [clock seconds]
      set combination [lindex $combinations $loop]
      #puts "[$bot name] Testing \"$combination\" ($count)"
      incr count
      set clone [$bot clone "$bot-clone"]
      set p $phase
      for { set i 0 } {$i < [llength $combination]} { incr i } {
          $clone program [lreplace [$clone program] \
                              $p $p [lindex $combination $i]]
          RTS::RunPhase $p $clone
          incr p
          if {![$clone placed]} { break }           
      }
      if {$bestProgram == ""} {
          set bestProgram [string trim [$clone program]]
          set bestCombo $combination
      }
      if {![$clone placed]} { 
          #puts "[$clone name] is dead ($count)"
          $clone destruct
          continue 
      }
      set score [RTSBotAI::ScoreMove $clone $bot $tx $ty \
                   $dt $face(x) $face(y)]
      if {($bestScore == "") || ($score > $bestScore)} {
          set bestProgram [string trim [$clone program]]
          set bestCombo $combination
          set bestScore $score
      }
      $clone destruct
      set sumTime [expr {$sumTime + [clock seconds] - $time}]
      if {$bestScore > 100} { break }
      if {[clock seconds] >= [expr {$startTime + 3}]} { break }
   }
   set cards [$bot cards]
   foreach card $bestCombo {
       set index [lsearch -exact $cards $card]
       if {$index == -1} { continue }
       set cards [lreplace $cards $index $index]
   }
   $bot program $bestProgram
   $bot cards $cards
   #puts "[$bot name] bestProgram scores $bestScore ($count) [expr $sumTime/double($count)] per"
}

proc RTSBotAI::GetNextItems { bot } {
    set x [$bot x]
    set y [$bot y]
    set dir [$bot face]
    incr x [RTS::GetDx $dir]
    incr y [RTS::GetDy $dir]
    if {![info exists RTS::map($x,$y)]} { return "" }
    return $RTS::map($x,$y)
}

proc RTSBotAI::IsNextItemFavorable {bot} {
    set items [RTSBotAI::GetNextItems $bot]
    if {$items == ""} { return 0 }
    foreach itemId $items {
        switch -- $RTS::item($itemId) {
            p { return -20 }
            w { return -10 }
            l { return -20 }
            r { return 10 }
            a { return 10 }
            b { return 20 }
        }        
    }
    return 5
}


proc RTSBotAI::ScoreMove { clone bot tx ty dt faceX faceY } {
    set score [expr {2.5 * ([$clone health] - [$bot health])}]
    set cx    [expr {$tx - [$clone x]}]
    set cy    [expr {$ty - [$clone y]}]
    set score [expr {$score + (5 * ($dt - (abs($cx) + abs($cy))))}]
    set score [expr {$score + [RTSBotAI::IsNextItemFavorable $clone]}]

    if { $cx == 0 } { 
        set score [expr {$score + 20}] 
        if {$faceX == [$clone face]} {
            set score [expr {$score + 30}]
        }
    }
    if { $cy == 0 } { 
        set score [expr {$score + 20}] 
        if {$faceY == [$clone face]} {
            set score [expr {$score + 30}] 
        }
    }
    if {([$clone x] == $tx) && ([$clone y] == $ty)} {
        set score [expr {$score + 100}]
    }
    return $score
}


proc RTSBotAI::PickCard { bot } {
    set x [$bot x]
    set y [$bot y]
    set dest [RTSBotAI::GetDestination $bot]
    set tx [lindex $dest 0]
    set ty [lindex $dest 1]
    set dx [expr {$tx - $x}]
    set dy [expr {$ty - $y}]
    set dt [expr {abs($dx) + abs($dy)}]
    if {$dx < 0} {
        set face(x) "w"
    } elseif {$dx > 0} { 
        set face(x) "e"
    } else {
        set face(x) "x"
    }
    if {$dy < 0} {
        set face(y) "n"
    } elseif {$dy > 0} {
        set face(y) "s"
    } else {
        set face(y) "x"
    }
    set face(bot) [$bot face]
    set face(90)  [RTS::GetRotatedDirection $face(bot) 90]
    set face(180) [RTS::GetRotatedDirection $face(bot) 180]
    set face(270) [RTS::GetRotatedDirection $face(bot) 270]


    set bestCardIndex ""
    set bestClone ""
    set bestScore ""
    foreach card [split [$bot cards]] {
        set cardIndex [$bot cardindex $card]
        if {$cardIndex == -1} { continue }
        set clone [$bot clone "$bot-clone"]
        $clone program [lreplace [split [$clone program]] \
                            $RTS::game(phase) $RTS::game(phase) $card]
        RTS::RunPhase $RTS::game(phase) $clone
        if {![$clone placed]} { 
            $clone destruct
            continue 
        }
        set score [RTSBotAI::ScoreMove $clone $bot $tx $ty \
                     $dt $face(x) $face(y)]

        #puts "Testing $bot for $card :$score ($bestScore)"        

        if {($bestCardIndex == "") || ($score > $bestScore)} { 
            RTSBotAI::SetBest
        }
        $clone destruct
    }
    if {$bestCardIndex == ""} {
        set bestCardIndex 0
    }
    set selectedCard [lindex [split [join [$bot cards]]] $bestCardIndex]

    #puts "Using $selectedCard for $bot"

    $bot cards [lreplace [split [join [$bot cards]]] \
                          $bestCardIndex $bestCardIndex]
    return $selectedCard
}

proc RTSBotAI::SetBest {} {
    uplevel { set bestCardIndex $cardIndex }
    uplevel { set bestClone $clone }
    uplevel { set bestScore $score }
}




proc RTSBotAI::GetDestination {bot} {
    switch -- $RTS::game(type) {
        dm      { set dest [RTSBotAI::GetAttackerDestination $bot] }
        ctf     { set dest [RTSBotAI::GetAttackerDestination $bot] }
        rly     { set dest [RTSBotAI::GetFlagDestination $bot] }
        default { set dest [RTSBotAI::GetAttackerDestination $bot] }
    }
    return $dest
}





proc RTSBotAI::GetFlagDestination { bot } {
    set neededFlag [lindex $RTS::game(rlyorder) [$bot nextflag]]
    if {$neededFlag == ""} { 
        return [RTSBotAI::GetAttackerDestination $bot] 
    }
    set itemId $RTS::map(flag,$neededFlag)
    return [list $RTS::item($itemId,x) $RTS::item($itemId,y)]
}






proc RTSBotAI::GetNavPointAtCoord { x y } {
    if {![info exists RTS::map($x,$y)] || ($RTS::map($x,$y) == "")} {
        return ""
    }
    foreach itemId $RTS::map($x,$y) {
        if {$RTS::item($itemId) == "n"} {
            return $itemId
        }
    }
    return ""
}



proc RTSBotAI::GetFlagDestinationWithApples { bot } {
    if {[info exists RTSBotAI::navpoint($bot)]
        && ($RTSBotAI::navpoint($bot) != "")} {
        return $RTSBotAI::navpoint($bot)
    }
    set neededFlag [lindex $RTS::game(rlyorder) [$bot nextflag]]
    if {$neededFlag == ""} { 
        return [RTSBotAI::GetAttackerDestination $bot] 
    }
    set itemId $RTS::map(flag,$neededFlag)
    set botNavItem [RTSBotAI::GetNavPointAtCoord [$bot x] [$bot y]]
    if {$botNavItem != ""} {
        puts "$bot on nav $RTS::item($botNavItem,id), item $botNavItem"
        if {[info exists RTSBotAI::navpoint($bot)]
            && ($RTSBotAI::navpoint($bot) == $botNavItem)} {
            unset RTSBotAI::navpoint($bot)
            puts "$bot reached destination $RTS::item($botNavItem,id), item $botNavItem"
        }
        set destNavItem [RTSBotAI::GetNavPointAtCoord \
                             $RTS::item($itemId,x) $RTS::item($itemId,y)]
        if {$destNavItem != ""} {
            puts "$bot dest nav $RTS::item($destNavItem,id), item $destNavItem"
            set dir [expr {$RTS::item($destNavItem,id) \
                         - $RTS::item($botNavItem,id)}]
            set dir [expr {$dir/abs($dir)}]
            set index [lsearch $RTS::map(navpoints) $botNavItem]
            if {$index != ""} {
                set nextNav [lindex $RTS::map(navpoints) \
                                 [expr {$index + $dir}]]
                puts "$bot next nav $nextNav ($RTS::map(navpoints))"
                if {$nextNav != ""} {
                    set itemId $nextNav
                    puts "$bot seeking nav $RTS::item($itemId,id), item $itemId @ $RTS::item($itemId,coords)"
                    set RTSBotAI::navpoint($bot) $itemId
                }
            }
        }
    }
    return [list $RTS::item($itemId,x) $RTS::item($itemId,y)]
}




proc RTSBotAI::GetAttackerDestination { bot } {
    variable kamakazi
    #
    # Go kamakazi ?
    #
    if {![info exists kamakazi($bot)] && ([expr {rand()}] < 0.5)} {
        set pickingTarget 1
        while {$pickingTarget} {
            set kamakazi($bot) \
                [lindex [RTS::RandomizeList [rtsbot bots]] \
                     [expr {int(rand() * [llength [rtsbot bots]])}]]
            if {$bot != $kamakazi($bot)} { 
                set pickingTarget 0
            }
        }
        set kamakazi($bot,turns) [expr {round(3 + (rand() * 5))}]
    }
    #
    # Determine Mode
    #
    if {[info exists kamakazi($bot)]} {
        set mode kamakazi
    } else {
        switch -- [expr {[$bot health]%10}] {
            0 { set mode agro }
            1 { set mode agro }
            2 { set mode retreat }
            3 { set mode retreat }
            4 { set mode retreat }
            5 { set mode normal }
            6 { set mode normal }
            7 { set mode normal }
            default { set mode agro }
        }
    }
    #
    # Determine potential targets
    #
    switch -- $mode {
        agro     { set targets [rtsbot bots]}
        retreat  { set targets $RTS::map(repairs)}
        normal   { set targets [concat [rtsbot bots] $RTS::map(repairs)]}
        kamakazi { set targets $kamakazi($bot)}
    }    
    set x [$bot x]
    set y [$bot y]
    set destX ""
    set destY ""
    set destT ""
    foreach target $targets {
        switch -glob -- $target {
            b* {
                if {$bot != $target} {
                    set tx [$target x] 
                    set ty [$target y]
                } else {
                    continue
                }
            }
            default {
                set tx $RTS::item($target,x)
                set ty $RTS::item($target,x)
            }
        }
        set dx [expr {$tx - $x}]
        set dy [expr {$ty - $y}]
        set dt [expr {abs($dx) + abs($dy)}]
        #puts "$bot @ $x,$y targets $target @ $tx,$ty x:$dx y:$dy total:$dt"
        if {(($destX == "") || ($destY == "") || ($destT == "")) 
            || ($dt < $destT)} {
            set destX $tx
            set destY $ty
            set destT $dt
            set deltaX $dx
            set deltaY $dy
        }
    }
    
    return [list $destX $destY]

}




