# Copyright 2001 Bryan Schofield
# This file is part of RoboTournament.
# 
# RoboTournament is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RoboTournament is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RoboTournament; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package provide RTSError 1.0


proc bgerror {err} {
   global errorInfo tcl_platform
   set w .__error__
   toplevel $w
   wm title $w "RoboTournmant Server Error"
   wm protocol $w WM_DELETE_WINDOW {}
   wm transient $w $w
   wm withdraw $w

   set f [frame $w.err]
   label $f.l \
      -text "RoboTournmant Server Error:" \
      -foreground "$::contrastForeground" \
      -anchor nw \
      -font "fnt-big"
   label $f.e \
      -text "$err" \
      -font "fnt-big" \
      -anchor w \
      -justify left \
      -wraplength 400
      
   grid $f.l -row 0 -col 0 -stick nw
   grid $f.e -row 1 -col 0 -stick ew
   grid columnconfigure $f 0 -weight 1
   
   set f [frame $w.trace \
            -highlightthickness 1 \
            -highlightbackground "black"]
   set t [text $f.t \
            -font "fnt-small" \
            -wrap none \
            -yscrollcommand "$f.ys set" \
            -xscrollcommand "$f.xs set" \
            -setgrid true \
            -width 50 \
            -height 10]
   scrollbar $f.ys \
      -command "$f.t yview" \
      -background "black" \
      -relief flat \
      -borderwidth 0 \
      -width 11
   scrollbar $f.xs \
      -command "$f.t xview" \
      -orient "horizontal" \
      -background "black"\
      -relief flat \
      -borderwidth 0 \
      -width 11
   grid $f.t  -row 0 -col 0 -stick news -padx 2 -pady 2
   grid $f.ys -row 0 -col 1 -stick ns  -padx 2 -pady 2
   grid $f.xs -row 1 -col 0 -stick ew -padx 2 -pady 2
   grid columnconfigure $f 0 -weight 1
   grid rowconfigure    $f 0 -weight 1
   
   
   set f [frame $w.butts]
   button $f.dismiss \
      -text "Dismiss" \
      -background "$::darkBackground" \
      -font "fnt-big" \
      -highlightbackground black \
      -highlightthickness 1 \
      -highlightcolor "black" \
      -command "destroy $w" \
      -activebackground "$::darkForeground"\
      -takefocus 0 \
      -borderwidth 1 \
      -width 7
      
    button $f.save \
      -text "Save Error To File" \
      -background "$::darkBackground" \
      -font "fnt-big" \
      -highlightbackground black \
      -highlightthickness 1 \
      -highlightcolor "black" \
      -command "destroy $w" \
      -activebackground "$::darkForeground"\
      -takefocus 0 \
      -borderwidth 1 \
      -command {
         set textwidget ".__error__.trace.t"
         set types {
            {{Text Files} .txt}
            {{Log Files} .log}
            {{All Files} *}
         }
         set filename [tk_getSaveFile \
                        -defaultextension .txt \
                        -filetypes $types \
                        -parent $textwidget]
         if {![string length $filename]} { return }
         if { [catch { open $filename w} f] } {
            tk_messageBox \
               -icon error \
               -title "RoboTournament Error" \
               -parent $textwidget \
               -message "An error occured saving the stack trace.\n$f"
            return
         }
         puts -nonewline $f [$textwidget get 1.0 end]
         close $f
      }
      
    button $f.quit \
      -text "Quit" \
      -background "$::darkBackground" \
      -font "fnt-big" \
      -highlightbackground black \
      -highlightthickness 1 \
      -highlightcolor "black" \
      -command "exit" \
      -activebackground "$::darkForeground"\
      -takefocus 0 \
      -borderwidth 1 \
      -width 7
      
   grid $f.dismiss $f.save $f.quit -padx 4 -pady 4 -stick ew
   grid columnconfigure $f {0 1 2} -weight 1
  
   
   grid $w.err -row 0 -col 0 -stick ew     -padx 4 -pady 4
   grid $w.trace -row 1 -col 0 -stick news -padx 8 -pady 4
   grid $w.butts -row 2 -col 0 -stick ew   -padx 4 -pady 4
   grid columnconfigure $w 0 -weight 1
   grid rowconfigure    $w 1 -weight 1


   #
   # Add the error text
   #

   $t insert end "RoboTournament Server Error\n"
   $t insert end "Version   $::version\n"
   $t insert end "Date      [clock format [clock seconds]]\n"
   $t insert end "Tcl/Tk    $::tk_patchLevel\n"
   $t insert end "Platform  $tcl_platform(platform)\n"
   $t insert end "\n** Trace Follows **\n\n"
   $t insert end $errorInfo


   #
   # Center the dialog window
   #
   update   
   set x [expr ([winfo screenwidth $w]  - [winfo reqwidth $w])/2]
   set y [expr ([winfo screenheight $w] - [winfo reqheight $w])/2]
   wm geometry $w +$x+$y
   wm deiconify $w
   
   grab $w
   tkwait window $w
   grab release $w
}
