# Copyright 2001 Bryan Schofield & Stephen Brazelton
# This file is part of RoboTournament.
# 
# RoboTournament is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RoboTournament is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RoboTournament; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package provide RTSInsult 0.1

#========================================================================
# rtsinsult
#
# Public level access to insults
#
proc ::rtsinsult {subcommand args} {
    set help "\nShould be:\n"
    append help "rtsinsult initialize\n"
    append help "rtsinsult insult <perpBot> <amtDamage> <victimBot>\n"


    switch -- $subcommand {
        initialize { 
            RTSInsult::Initialize
        }
        insult { 
            set perpBot [lindex $args 0]
            set amtDamage [lindex $args 1]
            set victimBot [lindex $args 2]
            RTSInsult::Insult $perpBot $amtDamage $victimBot
        }
        default {
            return -code error "Unknown command rtsinsult $subcommand $args $help"
        }
    }
}

namespace eval RTSInsult {
    variable insultsInitialized
    variable insultList
    variable rebuttalList
    variable chanceDivisor 1000
    set insultsInitialized 0
    set insultList {}
    set rebuttalList {}
}

#========================================================================
# RTSInsult::Initialize
#
# Initializes the insults from RTSInsults.txt
#
proc RTSInsult::Initialize {} {

    variable insultsInitialized

    if {$insultsInitialized} {return}

    variable insultList
    variable rebuttalList
    set list insultList

    if {[catch {open [file join $::libDir RTSInsult.cfg] r} insultChannel]} {
        return 0
    }
    while {[gets $insultChannel line] >= 0} {
        set line [string trim $line]
        if {[string length $line] == 0} { continue }
        if {[regexp {^\#} $line]} {continue}
        if {[string tolower $line] == "insult"} {
            set list insultList
        } elseif {[string tolower $line] == "rebuttal"} {
            set list rebuttalList
        } else {
            lappend $list $line
        }
    }
    set insultsInitialized 1
    close $insultChannel
    if {[llength $insultList] == 0} {
        lappend insultList "Your momma has bald tires."
    }
    if {[llength $rebuttalList] == 0} {
        lappend insultList "Ow, quit it!"
    }
}

#========================================================================
# RTSInsult::Insult
#
# 
#
proc RTSInsult::Insult {perpBot amtDamage victimBot} {
    variable insultsInitialized

    if {$insultsInitialized == 0} {
        RTSInsult::Initialize
    }

    set chance [expr $amtDamage/double($RTSInsult::chanceDivisor)]
    if {[$perpBot free] && ([expr rand()] < $chance)} {
        RTSInsult::BroadcastInsult $perpBot $amtDamage $victimBot insult
    }
    if {[$victimBot free] && ([expr rand()] < $chance)} {
        RTSInsult::BroadcastInsult $perpBot $amtDamage $victimBot rebuttal
    }
}


#========================================================================
# RTSInsult::BroadcastInsult
#
# 
#
proc RTSInsult::BroadcastInsult {perpBot amtDamage victimBot type} {

    if {$type == "insult"} {
        set index [expr int(rand() * [llength $RTSInsult::insultList])]
        set currentInsult \
            [lindex $RTSInsult::insultList $index]
        set botName [$perpBot name]
    } elseif {$type == "rebuttal"} {
        set index [expr int(rand() * [llength $RTSInsult::rebuttalList])]
        set currentInsult \
            [lindex $RTSInsult::rebuttalList $index]
        set botName [$victimBot name]
    }
    regsub {%v} $currentInsult [$victimBot name] currentInsult
    regsub {%p} $currentInsult [$perpBot name] currentInsult
    if {$currentInsult == ""} { 
       #puts "Empty insult: $perpBot $amtDamage $victimBot $type : $index"
       return 
    }
    rtssocket broadcast "CHAT <$botName> $currentInsult"
}
